@@ -2,10 +2,11 @@
  * ntp_request.h - definitions for the ntpd remote query facility
  */
 
-#ifndef _NTP_REQUEST_H
-#define _NTP_REQUEST_H
+#ifndef NTP_REQUEST_H
+#define NTP_REQUEST_H
 
 #include "ntp_types.h"
+#include "recvbuff.h"
 
 /*
  * A mode 7 packet is used exchanging data between an NTP server
@@ -381,7 +382,7 @@ struct info_peer {
 	u_int32 pkeyid;		/* unused */
 	u_int32 refid;		/* peer.refid */
 	u_int32 timer;		/* peer.timer */
-	s_fp rootdelay;		/* peer.distance */
+	s_fp rootdelay;		/* peer.delay */
 	u_fp rootdispersion;	/* peer.dispersion */
 	l_fp reftime;		/* peer.reftime */
 	l_fp org;		/* peer.org */
@@ -465,7 +466,7 @@ struct info_sys {
 	u_char leap;		/* system leap bits */
 	u_char stratum;		/* our stratum */
 	s_char precision;	/* local clock precision */
-	s_fp rootdelay;		/* distance from sync source */
+	s_fp rootdelay;		/* delay from sync source */
 	u_fp rootdispersion;	/* dispersion from sync source */
 	u_int32 refid;		/* reference ID of sync source */
 	l_fp reftime;		/* system reference time */
@@ -600,7 +601,6 @@ struct conf_peer {
 #define CONF_FLAG_IBURST	0x08
 #define CONF_FLAG_NOSELECT	0x10
 #define CONF_FLAG_SKEY		0x20
-#define CONF_FLAG_DYNAMIC	0x40
 
 /*
  * Structure for passing peer deletion information.  Currently
@@ -668,7 +668,7 @@ struct conf_restrict {
 struct info_monitor_1 {	
 	u_int32 lasttime;	/* last packet from this host */
 	u_int32 firsttime;	/* first time we received a packet */
-	u_int32 lastdrop;        /* last time we rejected a packet due to client limitation policy */
+	u_int32 restr;		/* restrict bits (was named lastdrop) */
 	u_int32 count;		/* count of packets received */
 	u_int32 addr;		/* host address V4 style */
 	u_int32 daddr;		/* destination host address */
@@ -689,7 +689,7 @@ struct info_monitor_1 {
 struct info_monitor {	
 	u_int32 lasttime;	/* last packet from this host */
 	u_int32 firsttime;	/* first time we received a packet */
-	u_int32 lastdrop;       /* last time we rejected a packet due to client limitation policy */
+	u_int32 restr;		/* restrict bits (was named lastdrop) */
 	u_int32 count;		/* count of packets received */
 	u_int32 addr;		/* host address */
 	u_short port;		/* port number of last reception */
@@ -701,7 +701,7 @@ struct info_monitor {
 };
 
 /*
- * Structure used for returning monitor data (old format
+ * Structure used for returning monitor data (old format)
  */
 struct old_info_monitor {	
 	u_int32 lasttime;	/* last packet from this host */
@@ -893,7 +893,6 @@ struct info_if_stats {
 	union addrun unmask;	        /* mask */
 	u_int32 v6_flag;                /* is this v6 */
 	char name[32];			/* name of interface */
-	u_short family;			/* Address family */
 	int32 flags;			/* interface flags */
 	int32 last_ttl;			/* last TTL specified */
 	int32 num_mcast;		/* No. of IP addresses in multicast socket */
@@ -904,9 +903,11 @@ struct info_if_stats {
 	u_int32 scopeid;		/* Scope used for Multicasting */
 	u_int32 ifindex;		/* interface index - from system */
 	u_int32 ifnum;		        /* sequential interface number */
-	u_char ignore_packets;	        /* Specify whether the packet should be ignored */
         u_int32 peercnt;		/* number of peers referencinf this interface - informational only */
+	u_short family;			/* Address family */
+	u_char ignore_packets;	        /* Specify whether the packet should be ignored */
         u_char action;		        /* reason the item is listed */
+	int32 _filler0;		        /* pad to a 64 bit size boundary */
 };
 
 #define IFS_EXISTS	1	/* just exists */
@@ -923,4 +924,10 @@ struct info_dns_assoc {
 	associd_t associd;	/* association ID */
 	char hostname[NTP_MAXHOSTNAME];	/* hostname */
 };
+
+/*
+ * function declarations
+ */
+int get_packet_mode(struct recvbuf *rbufp); /* Return packet mode */
+
 #endif /* NTP_REQUEST_H */
