@@ -1,12 +1,12 @@
 /*
- *  $Id: configfile.c,v 4.23 2006/07/01 21:57:23 bkorb Exp $
- *  Time-stamp:      "2006-07-01 12:46:31 bkorb"
+ *  $Id: configfile.c,v 1.21 2007/04/15 19:01:18 bkorb Exp $
+ *  Time-stamp:      "2007-04-15 11:22:46 bkorb"
  *
  *  configuration/rc/ini file handling.
  */
 
 /*
- *  Automated Options copyright 1992-2006 Bruce Korb
+ *  Automated Options copyright 1992-2007 Bruce Korb
  *
  *  Automated Options is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -53,7 +53,7 @@
 static void
 filePreset(
     tOptions*     pOpts,
-    const char*   pzFileName,
+    char const*   pzFileName,
     int           direction );
 
 static char*
@@ -113,7 +113,7 @@ skipUnknown( char* pzText );
 /*=export_func  configFileLoad
  *
  * what:  parse a configuration file
- * arg:   + const char*     + pzFile + the file to load +
+ * arg:   + char const*     + pzFile + the file to load +
  *
  * ret_type:  const tOptionValue*
  * ret_desc:  An allocated, compound value structure
@@ -141,17 +141,20 @@ skipUnknown( char* pzText );
  *  @end itemize
 =*/
 const tOptionValue*
-configFileLoad( const char* pzFile )
+configFileLoad( char const* pzFile )
 {
     tmap_info_t   cfgfile;
     tOptionValue* pRes = NULL;
+    tOptionLoadMode save_mode = option_load_mode;
+
     char* pzText =
         text_mmap( pzFile, PROT_READ, MAP_PRIVATE, &cfgfile );
 
     if (TEXT_MMAP_FAILED_ADDR(pzText))
         return NULL; /* errno is set */
 
-    pRes = optionLoadNested(pzText, pzFile, strlen(pzFile), OPTION_LOAD_COOKED);
+    option_load_mode = OPTION_LOAD_COOKED;
+    pRes = optionLoadNested(pzText, pzFile, strlen(pzFile));
 
     if (pRes == NULL) {
         int err = errno;
@@ -159,6 +162,8 @@ configFileLoad( const char* pzFile )
         errno = err;
     } else
         text_munmap( &cfgfile );
+
+    option_load_mode = save_mode;
     return pRes;
 }
 
@@ -167,8 +172,8 @@ configFileLoad( const char* pzFile )
  *
  * what:  find a hierarcicaly valued option instance
  * arg:   + const tOptDesc* + pOptDesc + an option with a nested arg type +
- * arg:   + const char*     + name     + name of value to find +
- * arg:   + const char*     + value    + the matching value    +
+ * arg:   + char const*     + name     + name of value to find +
+ * arg:   + char const*     + value    + the matching value    +
  *
  * ret_type:  const tOptionValue*
  * ret_desc:  a compound value structure
@@ -189,7 +194,7 @@ configFileLoad( const char* pzFile )
 =*/
 const tOptionValue*
 optionFindValue( const tOptDesc* pOptDesc,
-                 const char* pzName, const char* pzVal )
+                 char const* pzName, char const* pzVal )
 {
     const tOptionValue* pRes = NULL;
 
@@ -204,9 +209,8 @@ optionFindValue( const tOptDesc* pOptDesc,
 
     else do {
         tArgList* pAL = pOptDesc->optCookie;
-        int ct = pAL->useCt;
-        const tOptionValue** ppOV =
-            (const tOptionValue**)(void*)&(pAL->apzArgs);
+        int    ct   = pAL->useCt;
+        void** ppOV = (void**)(pAL->apzArgs);
 
         if (ct == 0) {
             errno = ENOENT;
@@ -214,7 +218,7 @@ optionFindValue( const tOptDesc* pOptDesc,
         }
 
         if (pzName == NULL) {
-            pRes = *ppOV;
+            pRes = (tOptionValue*)*ppOV;
             break;
         }
 
@@ -243,8 +247,8 @@ optionFindValue( const tOptDesc* pOptDesc,
  * what:  find a hierarcicaly valued option instance
  * arg:   + const tOptDesc* + pOptDesc + an option with a nested arg type +
  * arg:   + const tOptionValue* + pPrevVal + the last entry +
- * arg:   + const char*     + name     + name of value to find +
- * arg:   + const char*     + value    + the matching value    +
+ * arg:   + char const*     + name     + name of value to find +
+ * arg:   + char const*     + value    + the matching value    +
  *
  * ret_type:  const tOptionValue*
  * ret_desc:  a compound value structure
@@ -266,7 +270,7 @@ optionFindValue( const tOptDesc* pOptDesc,
 =*/
 const tOptionValue*
 optionFindNextValue( const tOptDesc* pOptDesc, const tOptionValue* pPrevVal,
-                 const char* pzName, const char* pzVal )
+                 char const* pzName, char const* pzVal )
 {
     int foundOldVal = 0;
     tOptionValue* pRes = NULL;
@@ -282,8 +286,8 @@ optionFindNextValue( const tOptDesc* pOptDesc, const tOptionValue* pPrevVal,
 
     else do {
         tArgList* pAL = pOptDesc->optCookie;
-        int ct = pAL->useCt;
-        tOptionValue** ppOV = (tOptionValue**)(void*)&(pAL->apzArgs);
+        int    ct   = pAL->useCt;
+        void** ppOV = (void**)pAL->apzArgs;
 
         if (ct == 0) {
             errno = ENOENT;
@@ -311,7 +315,7 @@ optionFindNextValue( const tOptDesc* pOptDesc, const tOptionValue* pPrevVal,
  *
  * what:  get a specific value from a hierarcical list
  * arg:   + const tOptionValue* + pOptValue + a hierarchcal value +
- * arg:   + const char*   + valueName + name of value to get +
+ * arg:   + char const*   + valueName + name of value to get +
  *
  * ret_type:  const tOptionValue*
  * ret_desc:  a compound value structure
@@ -333,7 +337,7 @@ optionFindNextValue( const tOptDesc* pOptDesc, const tOptionValue* pPrevVal,
  *  @end itemize
 =*/
 const tOptionValue*
-optionGetValue( const tOptionValue* pOld, const char* pzValName )
+optionGetValue( const tOptionValue* pOld, char const* pzValName )
 {
     tArgList*     pAL;
     tOptionValue* pRes = NULL;
@@ -345,11 +349,11 @@ optionGetValue( const tOptionValue* pOld, const char* pzValName )
     pAL = pOld->v.nestVal;
 
     if (pAL->useCt > 0) {
-        int ct = pAL->useCt;
-        tOptionValue** papOV = (tOptionValue**)(pAL->apzArgs);
+        int    ct    = pAL->useCt;
+        void** papOV = (void**)(pAL->apzArgs);
 
         if (pzValName == NULL) {
-            pRes = *papOV;
+            pRes = (tOptionValue*)*papOV;
         }
 
         else do {
@@ -393,8 +397,8 @@ optionGetValue( const tOptionValue* pOld, const char* pzValName )
  *  @code{ENOENT} - the supplied @code{pOldValue} pointed to the last entry.
  *  @end itemize
 =*/
-const tOptionValue*
-optionNextValue( const tOptionValue* pOVList, const tOptionValue* pOldOV )
+tOptionValue const *
+optionNextValue(tOptionValue const * pOVList,tOptionValue const * pOldOV )
 {
     tArgList*     pAL;
     tOptionValue* pRes = NULL;
@@ -406,8 +410,8 @@ optionNextValue( const tOptionValue* pOVList, const tOptionValue* pOldOV )
     }
     pAL = pOVList->v.nestVal;
     {
-        int   ct   = pAL->useCt;
-        tOptionValue** papNV = (tOptionValue**)(pAL->apzArgs);
+        int    ct    = pAL->useCt;
+        void** papNV = (void**)(pAL->apzArgs);
 
         while (ct-- > 0) {
             tOptionValue* pNV = *(papNV++);
@@ -417,7 +421,7 @@ optionNextValue( const tOptionValue* pOVList, const tOptionValue* pOldOV )
 
                 } else {
                     err  = 0;
-                    pRes = *papNV;
+                    pRes = (tOptionValue*)*papNV;
                 }
                 break;
             }
@@ -436,13 +440,13 @@ optionNextValue( const tOptionValue* pOVList, const tOptionValue* pOldOV )
 static void
 filePreset(
     tOptions*     pOpts,
-    const char*   pzFileName,
+    char const*   pzFileName,
     int           direction )
 {
     tmap_info_t   cfgfile;
+    tOptState     st = OPTSTATE_INITIALIZER(PRESET);
     char*         pzFileText =
         text_mmap( pzFileName, PROT_READ|PROT_WRITE, MAP_PRIVATE, &cfgfile );
-    tOptState     st = OPTSTATE_INITIALIZER(PRESET);
 
     if (TEXT_MMAP_FAILED_ADDR(pzFileText))
         return;
@@ -462,14 +466,14 @@ filePreset(
         st.flags = OPTST_SET;
 
     do  {
-        while (isspace( *pzFileText ))  pzFileText++;
+        while (isspace( (int)*pzFileText ))  pzFileText++;
 
-        if (isalpha( *pzFileText )) {
+        if (isalpha( (int)*pzFileText )) {
             pzFileText = handleConfig( pOpts, &st, pzFileText, direction );
 
         } else switch (*pzFileText) {
         case '<':
-            if (isalpha( pzFileText[1] ))
+            if (isalpha( (int)pzFileText[1] ))
                 pzFileText = handleStructure(pOpts, &st, pzFileText, direction);
 
             else switch (pzFileText[1]) {
@@ -542,8 +546,11 @@ handleConfig(
     char* pzName = pzText++;
     char* pzEnd  = strchr( pzText, '\n' );
 
-    while (ISNAMECHAR( *pzText ))  pzText++;
-    while (isspace( *pzText )) pzText++;
+    if (pzEnd == NULL)
+        return pzText + strlen(pzText);
+
+    while (ISNAMECHAR( (int)*pzText ))  pzText++;
+    while (isspace( (int)*pzText )) pzText++;
     if (pzText > pzEnd) {
     name_only:
         *pzEnd++ = NUL;
@@ -557,10 +564,10 @@ handleConfig(
      *  is an invalid format and we give up parsing the text.
      */
     if ((*pzText == '=') || (*pzText == ':')) {
-        while (isspace( *++pzText ))   ;
+        while (isspace( (int)*++pzText ))   ;
         if (pzText > pzEnd)
             goto name_only;
-    } else if (! isspace(pzText[-1]))
+    } else if (! isspace((int)pzText[-1]))
         return NULL;
 
     /*
@@ -624,7 +631,7 @@ handleDirective(
     size_t name_len;
 
     if (  (strncmp( pzText+2, zProg, title_len ) != 0)
-       || (! isspace( pzText[title_len+2] )) )  {
+       || (! isspace( (int)pzText[title_len+2] )) )  {
         pzText = strchr( pzText+2, '>' );
         if (pzText != NULL)
             pzText++;
@@ -638,8 +645,8 @@ handleDirective(
     do  {
         pzText += title_len;
 
-        if (isspace(*pzText)) {
-            while (isspace(*pzText))  pzText++;
+        if (isspace((int)*pzText)) {
+            while (isspace((int)*pzText))  pzText++;
             if (  (strneqvcmp( pzText, pOpts->pzProgName, (int)name_len) == 0)
                && (pzText[name_len] == '>'))  {
                 pzText += name_len + 1;
@@ -698,10 +705,11 @@ handleStructure(
     char*         pzText,
     int           direction )
 {
-    tOptionLoadMode mode = OPTION_LOAD_UNCOOKED;
+    tOptionLoadMode mode = option_load_mode;
     tOptionValue     valu;
 
     char* pzName = ++pzText;
+    char* pzData;
     char* pcNulPoint;
 
     while (ISNAMECHAR( *pzText ))  pzText++;
@@ -716,13 +724,14 @@ handleStructure(
             break;
         if (*pzText != '/')
             return NULL;
+        /* FALLTHROUGH */
 
     case '/':
         if (pzText[1] != '>')
             return NULL;
         *pzText = NUL;
         pzText += 2;
-        loadOptionLine( pOpts, pOS, pzName, direction, OPTION_LOAD_KEEP );
+        loadOptionLine( pOpts, pOS, pzName, direction, mode );
         return pzText;
 
     case '>':
@@ -736,10 +745,11 @@ handleStructure(
     }
 
     /*
-     *  If we are here, we have a value.  Separate the name from the
-     *  value for a moment.
+     *  If we are here, we have a value.  "pzText" points to a closing angle
+     *  bracket.  Separate the name from the value for a moment.
      */
     *pcNulPoint = NUL;
+    pzData = ++pzText;
 
     /*
      *  Find the end of the option text and NUL terminate it
@@ -753,7 +763,7 @@ handleStructure(
         sprintf( pz, "</%s>", pzName );
         *pzText = ' ';
         pzText = strstr( pzText, pz );
-        if (pz != z) free(pz);
+        if (pz != z) AGFREE(pz);
 
         if (pzText == NULL)
             return pzText;
@@ -765,8 +775,9 @@ handleStructure(
 
     /*
      *  Rejoin the name and value for parsing by "loadOptionLine()".
+     *  Erase any attributes parsed by "parseAttributes()".
      */
-    *(pcNulPoint++) = ' ';
+    memset(pcNulPoint, ' ', pzData - pcNulPoint);
 
     /*
      *  "pzName" points to what looks like text for one option/configurable.
@@ -789,7 +800,7 @@ internalFileLoad( tOptions* pOpts )
 {
     int     idx;
     int     inc = DIRECTION_PRESET;
-    char    zFileName[ MAXPATHLEN+1 ];
+    char    zFileName[ AG_PATH_MAX+1 ];
 
     if (pOpts->papzHomeList == NULL)
         return;
@@ -827,7 +838,7 @@ internalFileLoad( tOptions* pOpts )
 
         idx += inc;
 
-        if (! optionMakePath( zFileName, sizeof( zFileName ),
+        if (! optionMakePath( zFileName, (int)sizeof(zFileName),
                               pzPath, pOpts->pzProgPath ))
             continue;
 
@@ -858,11 +869,14 @@ internalFileLoad( tOptions* pOpts )
          *  IF we are now to skip config files AND we are presetting,
          *  THEN change direction.  We must go the other way.
          */
-        if (SKIP_RC_FILES(pOpts) && PRESETTING(inc)) {
-            idx -= inc;  /* go back and reprocess current file */
-            inc =  DIRECTION_PROCESS;
+        {
+            tOptDesc * pOD = pOpts->pOptDesc + pOpts->specOptIdx.save_opts+1;
+            if (DISABLED_OPT(pOD) && PRESETTING(inc)) {
+                idx -= inc;  /* go back and reprocess current file */
+                inc =  DIRECTION_PROCESS;
+            }
         }
-    } /* For every path in the home list, ... */
+    } /* twice for every path in the home list, ... */
 }
 
 
@@ -871,7 +885,7 @@ internalFileLoad( tOptions* pOpts )
  * what: Load the locatable config files, in order
  *
  * arg:  + tOptions*   + pOpts  + program options descriptor +
- * arg:  + const char* + pzProg + program name +
+ * arg:  + char const* + pzProg + program name +
  *
  * ret_type:  int
  * ret_desc:  0 -> SUCCESS, -1 -> FAILURE
@@ -900,7 +914,7 @@ internalFileLoad( tOptions* pOpts )
  *       always be returned.
 =*/
 int
-optionFileLoad( tOptions* pOpts, const char* pzProgram )
+optionFileLoad( tOptions* pOpts, char const* pzProgram )
 {
     if (! SUCCESSFUL( validateOptionsStruct( pOpts, pzProgram )))
         return -1;
@@ -919,27 +933,27 @@ optionFileLoad( tOptions* pOpts, const char* pzProgram )
  * arg:   + tOptDesc* + pOptDesc + the descriptor for this arg +
  *
  * doc:
- *  Processes the options found in the file named with pOptDesc->pzLastArg.
+ *  Processes the options found in the file named with
+ *  pOptDesc->optArg.argString.
 =*/
 void
 optionLoadOpt( tOptions* pOpts, tOptDesc* pOptDesc )
 {
     /*
-     *  IF the option is not being disabled,
-     *  THEN load the file.  There must be a file.
-     *  (If it is being disabled, then the disablement processing
-     *  already took place.  It must be done to suppress preloading
-     *  of ini/rc files.)
+     *  IF the option is not being disabled, THEN load the file.  There must
+     *  be a file.  (If it is being disabled, then the disablement processing
+     *  already took place.  It must be done to suppress preloading of ini/rc
+     *  files.)
      */
     if (! DISABLED_OPT( pOptDesc )) {
         struct stat sb;
-        if (stat( pOptDesc->pzLastArg, &sb ) != 0) {
+        if (stat( pOptDesc->optArg.argString, &sb ) != 0) {
             if ((pOpts->fOptSet & OPTPROC_ERRSTOP) == 0)
                 return;
 
             fprintf( stderr, zFSErrOptLoad, errno, strerror( errno ),
-                     pOptDesc->pzLastArg );
-            (*pOpts->pUsageProc)( pOpts, EXIT_FAILURE );
+                     pOptDesc->optArg.argString );
+            exit(EX_NOINPUT);
             /* NOT REACHED */
         }
 
@@ -947,12 +961,12 @@ optionLoadOpt( tOptions* pOpts, tOptDesc* pOptDesc )
             if ((pOpts->fOptSet & OPTPROC_ERRSTOP) == 0)
                 return;
 
-            fprintf( stderr, zNotFile, pOptDesc->pzLastArg );
-            (*pOpts->pUsageProc)( pOpts, EXIT_FAILURE );
+            fprintf( stderr, zNotFile, pOptDesc->optArg.argString );
+            exit(EX_NOINPUT);
             /* NOT REACHED */
         }
 
-        filePreset(pOpts, pOptDesc->pzLastArg, DIRECTION_CALLED);
+        filePreset(pOpts, pOptDesc->optArg.argString, DIRECTION_CALLED);
     }
 }
 
@@ -989,7 +1003,7 @@ parseAttributes(
             break;
         }
 
-        while (isspace( *++pzText ))   ;
+        while (isspace( (int)*++pzText ))   ;
 
         if (strncmp( pzText, zLoadType, lenLoadType ) == 0) {
             pzText = parseValueType( pzText+lenLoadType, pType );
@@ -1044,7 +1058,7 @@ parseLoadMode(
         if (strncmp( pzText, zLoadCooked, len ) == 0) {
             if (  (pzText[len] == '>')
                || (pzText[len] == '/')
-               || isspace(pzText[len])) {
+               || isspace((int)pzText[len])) {
                 *pMode = OPTION_LOAD_COOKED;
                 return pzText + len;
             }
@@ -1057,7 +1071,7 @@ parseLoadMode(
         if (strncmp( pzText, zLoadUncooked, len ) == 0) {
             if (  (pzText[len] == '>')
                || (pzText[len] == '/')
-               || isspace(pzText[len])) {
+               || isspace((int)pzText[len])) {
                 *pMode = OPTION_LOAD_UNCOOKED;
                 return pzText + len;
             }
@@ -1070,7 +1084,7 @@ parseLoadMode(
         if (strncmp( pzText, zLoadKeep, len ) == 0) {
             if (  (pzText[len] == '>')
                || (pzText[len] == '/')
-               || isspace(pzText[len])) {
+               || isspace((int)pzText[len])) {
                 *pMode = OPTION_LOAD_KEEP;
                 return pzText + len;
             }
@@ -1111,7 +1125,7 @@ parseValueType(
     {
         size_t len = strlen(zLtypeString);
         if (strncmp( pzText, zLtypeString, len ) == 0) {
-            if ((pzText[len] == '>') || isspace(pzText[len])) {
+            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
                 pType->valType = OPARG_TYPE_STRING;
                 return pzText + len;
             }
@@ -1122,7 +1136,7 @@ parseValueType(
     {
         size_t len = strlen(zLtypeInteger);
         if (strncmp( pzText, zLtypeInteger, len ) == 0) {
-            if ((pzText[len] == '>') || isspace(pzText[len])) {
+            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
                 pType->valType = OPARG_TYPE_NUMERIC;
                 return pzText + len;
             }
@@ -1144,7 +1158,7 @@ parseValueType(
     {
         size_t len = strlen(zLtypeKeyword);
         if (strncmp( pzText, zLtypeKeyword, len ) == 0) {
-            if ((pzText[len] == '>') || isspace(pzText[len])) {
+            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
                 pType->valType = OPARG_TYPE_ENUMERATION;
                 return pzText + len;
             }
@@ -1155,7 +1169,7 @@ parseValueType(
     {
         size_t len = strlen(zLtypeSetMembership);
         if (strncmp( pzText, zLtypeSetMembership, len ) == 0) {
-            if ((pzText[len] == '>') || isspace(pzText[len])) {
+            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
                 pType->valType = OPARG_TYPE_MEMBERSHIP;
                 return pzText + len;
             }
@@ -1166,7 +1180,7 @@ parseValueType(
     {
         size_t len = strlen(zLtypeNest);
         if (strncmp( pzText, zLtypeNest, len ) == 0) {
-            if ((pzText[len] == '>') || isspace(pzText[len])) {
+            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
                 pType->valType = OPARG_TYPE_HIERARCHY;
                 return pzText + len;
             }
@@ -1188,7 +1202,7 @@ static char*
 skipUnknown( char* pzText )
 {
     for (;; pzText++) {
-        if (isspace( *pzText ))  return pzText;
+        if (isspace( (int)*pzText ))  return pzText;
         switch (*pzText) {
         case NUL: return NULL;
         case '/':
@@ -1207,11 +1221,11 @@ skipUnknown( char* pzText )
  *  already been called.)
  */
 LOCAL tSuccess
-validateOptionsStruct( tOptions* pOpts, const char* pzProgram )
+validateOptionsStruct( tOptions* pOpts, char const* pzProgram )
 {
     if (pOpts == NULL) {
         fputs( zAO_Bad, stderr );
-        exit( EXIT_FAILURE );
+        exit( EX_CONFIG );
     }
 
     /*
@@ -1249,7 +1263,7 @@ validateOptionsStruct( tOptions* pOpts, const char* pzProgram )
      *  and the set of equivalent characters.
      */
     if (pOpts->pzProgName == NULL) {
-        const char* pz = strrchr( pzProgram, DIRCH );
+        char const* pz = strrchr( pzProgram, DIRCH );
 
         if (pz == NULL)
              pOpts->pzProgName = pzProgram;
@@ -1271,7 +1285,6 @@ validateOptionsStruct( tOptions* pOpts, const char* pzProgram )
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
- * tab-width: 4
  * indent-tabs-mode: nil
  * End:
  * end of autoopts/configfile.c */
