@@ -38,6 +38,9 @@
 #ifdef HAVE_SYS_SOCKIO_H	/* UXPV: SIOC* #defines (Frank Vance <fvance@waii.com>) */
 # include <sys/sockio.h>
 #endif
+#ifdef HAVE_SYS_UIO_H
+# include <sys/uio.h>
+#endif
 
 /*
  * setsockopt does not always have the same arg declaration
@@ -132,6 +135,8 @@ struct interface *loopback_interface;	/* loopback ipv4 interface */
 
 int ninterfaces;			/* Total number of interfaces */
 
+volatile int disable_dynamic_updates;   /* when set to != 0 dynamic updates won't happen */
+
 #ifdef REFCLOCK
 /*
  * Refclock stuff.	We keep a chain of structures with data concerning
@@ -165,28 +170,28 @@ int maxactivefd;
  */
 static  u_char          sys_interphase = 0;
 
-static  struct interface *new_interface P((struct interface *));
-static  void add_interface P((struct interface *));
-static  void update_interfaces P((u_short, interface_receiver_t, void *));
-static  void remove_interface P((struct interface *));
-static  struct interface *create_interface P((u_short, struct interface *));
+static  struct interface *new_interface (struct interface *);
+static  void add_interface (struct interface *);
+static  void update_interfaces (u_short, interface_receiver_t, void *);
+static  void remove_interface (struct interface *);
+static  struct interface *create_interface (u_short, struct interface *);
 
-static int	move_fd		P((SOCKET));
+static int	move_fd		(SOCKET);
 
 /*
  * Multicast functions
  */
-static	isc_boolean_t	addr_ismulticast	 P((struct sockaddr_storage *));
+static	isc_boolean_t	addr_ismulticast	 (struct sockaddr_storage *);
 /*
  * Not all platforms support multicast
  */
 #ifdef MCAST
-static	isc_boolean_t	socket_multicast_enable	 P((struct interface *, int, struct sockaddr_storage *));
-static	isc_boolean_t	socket_multicast_disable P((struct interface *, struct sockaddr_storage *));
+static	isc_boolean_t	socket_multicast_enable	 (struct interface *, int, struct sockaddr_storage *);
+static	isc_boolean_t	socket_multicast_disable (struct interface *, struct sockaddr_storage *);
 #endif
 
 #ifdef DEBUG
-static void print_interface	P((struct interface *, char *, char *));
+static void print_interface	(struct interface *, char *, char *);
 #define DPRINT_INTERFACE(_LVL_, _ARGS_) do { if (debug >= (_LVL_)) { print_interface _ARGS_; } } while (0)
 #else
 #define DPRINT_INTERFACE(_LVL_, _ARGS_)
@@ -218,21 +223,21 @@ struct asyncio_reader {
 
 ISC_LIST(struct asyncio_reader) asyncio_reader_list;
 
-static void delete_asyncio_reader P((struct asyncio_reader *));
-static struct asyncio_reader *new_asyncio_reader P((void));
-static void add_asyncio_reader P((struct asyncio_reader *, enum desc_type));
-static void remove_asyncio_reader P((struct asyncio_reader *));
+static void delete_asyncio_reader (struct asyncio_reader *);
+static struct asyncio_reader *new_asyncio_reader (void);
+static void add_asyncio_reader (struct asyncio_reader *, enum desc_type);
+static void remove_asyncio_reader (struct asyncio_reader *);
 
 #endif /* !defined(HAVE_IO_COMPLETION_PORT) && defined(HAS_ROUTING_SOCKET) */
 
-static void init_async_notifications P((void));
+static void init_async_notifications (void);
 
-static	int create_sockets	P((u_short));
-static	SOCKET	open_socket	P((struct sockaddr_storage *, int, int, struct interface *));
-static	char *	fdbits		P((int, fd_set *));
-static	void	set_reuseaddr	P((int));
-static	isc_boolean_t	socket_broadcast_enable	 P((struct interface *, SOCKET, struct sockaddr_storage *));
-static	isc_boolean_t	socket_broadcast_disable P((struct interface *, struct sockaddr_storage *));
+static	int create_sockets	(u_short);
+static	SOCKET	open_socket	(struct sockaddr_storage *, int, int, struct interface *);
+static	char *	fdbits		(int, fd_set *);
+static	void	set_reuseaddr	(int);
+static	isc_boolean_t	socket_broadcast_enable	 (struct interface *, SOCKET, struct sockaddr_storage *);
+static	isc_boolean_t	socket_broadcast_disable (struct interface *, struct sockaddr_storage *);
 
 ISC_LIST(vsock_t)	fd_list;
 
@@ -251,24 +256,25 @@ ISC_LIST(struct interface)     inter_list;
 static struct interface *wildipv4 = NULL;
 static struct interface *wildipv6 = NULL;
 
-void	add_fd_to_list	P((SOCKET, enum desc_type));
-void	close_and_delete_fd_from_list	P((SOCKET));
-void	add_addr_to_list	P((struct sockaddr_storage *, struct interface *));
-void	delete_addr_from_list	P((struct sockaddr_storage *));
-struct interface *find_addr_in_list	P((struct sockaddr_storage *));
-struct interface *find_flagged_addr_in_list P((struct sockaddr_storage *, int));
-static void create_wildcards	P((u_short));
-isc_boolean_t address_okay	P((isc_interface_t *));
-void	convert_isc_if		P((isc_interface_t *, struct interface *, u_short));
-static struct interface *findlocalinterface	P((struct sockaddr_storage *));
-static struct interface *findlocalcastinterface	P((struct sockaddr_storage *, int));
+static void	add_fd_to_list	(SOCKET, enum desc_type);
+static void	close_and_delete_fd_from_list	(SOCKET);
+static void	add_addr_to_list	(struct sockaddr_storage *, struct interface *);
+static void	delete_addr_from_list	(struct sockaddr_storage *);
+static struct interface *find_addr_in_list	(struct sockaddr_storage *);
+static struct interface *find_flagged_addr_in_list (struct sockaddr_storage *, int);
+static void	create_wildcards	(u_short);
+static isc_boolean_t	address_okay	(isc_interface_t *);
+static void		convert_isc_if		(isc_interface_t *, struct interface *, u_short);
+static struct interface *getinterface	(struct sockaddr_storage *);
+static struct interface *findlocalinterface	(struct sockaddr_storage *);
+static struct interface *findlocalcastinterface	(struct sockaddr_storage *, int);
 
 /*
  * Routines to read the ntp packets
  */
 #if !defined(HAVE_IO_COMPLETION_PORT)
-static inline int     read_network_packet	P((SOCKET, struct interface *, l_fp));
-static inline int     read_refclock_packet	P((SOCKET, struct refclockio *, l_fp));
+static inline int     read_network_packet	(SOCKET, struct interface *, l_fp);
+static inline int     read_refclock_packet	(SOCKET, struct refclockio *, l_fp);
 #endif
 
 #ifdef SYS_WINNT
@@ -813,7 +819,7 @@ list_if_listening(struct interface *interface, u_short port)
 		"Enabled": "Disabled");
 }
 
-void
+static void
 create_wildcards(u_short port) {
 	isc_boolean_t okipv4 = ISC_TRUE;
 	/*
@@ -850,6 +856,8 @@ create_wildcards(u_short port) {
 		if (interface->fd != INVALID_SOCKET) {
 			wildipv4 = interface;
 			any_interface = interface;
+			
+			add_addr_to_list(&interface->sin, interface);
 			add_interface(interface);
 			list_if_listening(interface, port);
 		} else {
@@ -885,6 +893,7 @@ create_wildcards(u_short port) {
 		if (interface->fd != INVALID_SOCKET) {
 			wildipv6 = interface;
 			any6_interface = interface;
+			add_addr_to_list(&interface->sin, interface);
 			add_interface(interface);
 			list_if_listening(interface, port);
 		} else {
@@ -897,7 +906,7 @@ create_wildcards(u_short port) {
 }
 
 
-isc_boolean_t
+static isc_boolean_t
 address_okay(isc_interface_t *isc_if) {
 
 	DPRINTF(4, ("address_okay: listen Virtual: %d, IF name: %s, Up Flag: %d\n", 
@@ -934,7 +943,7 @@ address_okay(isc_interface_t *isc_if) {
 	return (ISC_TRUE);
 }
 
-void
+static void
 convert_isc_if(isc_interface_t *isc_if, struct interface *itf, u_short port) {
 	itf->scopeid = 0;
 	itf->family = (short) isc_if->af;
@@ -1034,9 +1043,11 @@ refresh_interface(struct interface * interface)
 void
 interface_update(interface_receiver_t receiver, void *data)
 {
-  	BLOCKIO();
-	update_interfaces(htons(NTP_PORT), receiver, data);
-  	UNBLOCKIO();
+	if (!disable_dynamic_updates) {
+		BLOCKIO();
+		update_interfaces(htons(NTP_PORT), receiver, data);
+		UNBLOCKIO();
+	}
 }
 
 /*
@@ -1044,15 +1055,15 @@ interface_update(interface_receiver_t receiver, void *data)
  * a wildcard address
  */
 static int
-is_wildcard_ifaddr(struct interface *itf)
+is_wildcard_addr(struct sockaddr_storage *sas)
 {
-	if (itf->family == AF_INET &&
-	    ((struct sockaddr_in*)&itf->sin)->sin_addr.s_addr == htonl(INADDR_ANY))
+	if (sas->ss_family == AF_INET &&
+	    ((struct sockaddr_in*)sas)->sin_addr.s_addr == htonl(INADDR_ANY))
 		return 1;
 
 #ifdef INCLUDE_IPV6_SUPPORT
-	if (itf->family == AF_INET6 &&
-	    memcmp(&((struct sockaddr_in6*)&itf->sin)->sin6_addr, &in6addr_any,
+	if (sas->ss_family == AF_INET6 &&
+	    memcmp(&((struct sockaddr_in6*)sas)->sin6_addr, &in6addr_any,
 		   sizeof(in6addr_any) == 0))
 		return 1;
 #endif
@@ -1060,6 +1071,49 @@ is_wildcard_ifaddr(struct interface *itf)
 	return 0;
 }
 
+#ifdef OS_NEEDS_REUSEADDR_FOR_IFADDRBIND
+/*
+ * enable/disable re-use of wildcard address socket
+ */
+static void
+set_wildcard_reuse(int family, int on)
+{
+	int onvalue = 1;
+	int offvalue = 0;
+	int *onoff;
+	SOCKET fd = INVALID_SOCKET;
+
+	onoff = on ? &onvalue : &offvalue;
+
+	switch (family) {
+	case AF_INET:
+		if (any_interface) {
+			fd = any_interface->fd;
+		}
+		break;
+
+#ifdef INCLUDE_IPV6_SUPPORT
+	case AF_INET6:
+		if (any6_interface) {
+			fd = any6_interface->fd;
+		}
+		break;
+#endif /* !INCLUDE_IPV6_SUPPORT */
+	}
+
+	if (fd != INVALID_SOCKET) {
+		if (setsockopt(fd, SOL_SOCKET,
+			       SO_REUSEADDR, (char *)onoff,
+			       sizeof(*onoff))) {
+			netsyslog(LOG_ERR, "set_wildcard_reuse: setsockopt(SO_REUSEADDR, %s) failed: %m", *onoff ? "on" : "off");
+		}
+		DPRINTF(4, ("set SO_REUSEADDR to %s on %s\n", *onoff ? "ON" : "OFF",
+			    stoa((family == AF_INET) ?
+				  &any_interface->sin : &any6_interface->sin)));
+	}
+}
+#endif /* OS_NEEDS_REUSEADDR_FOR_IFADDRBIND */
+
 /*
  * update_interface strategy
  *
@@ -1074,7 +1128,7 @@ is_wildcard_ifaddr(struct interface *itf)
  *     attempt to create a new interface entry
  *
  * Phase 2:
- * forall currently known interfaces
+ * forall currently known non MCAST and WILDCARD interfaces
  *   if interface does not match configuration phase (not seen in phase 1):
  *     remove interface from known interface list
  *     forall peers associated with this interface
@@ -1180,7 +1234,7 @@ update_interfaces(
 		 * address - some dhcp clients produce that in the
 		 * wild
 		 */
-		if (is_wildcard_ifaddr(&interface))
+		if (is_wildcard_addr(&interface.sin))
 			continue;
 
 		/*
@@ -1189,7 +1243,7 @@ update_interfaces(
 		 * with the appropriate socket (our name space is
 		 * (ip-address) - NOT (interface name, ip-address))
 		 */
-		iface = findlocalinterface(&interface.sin);
+		iface = getinterface(&interface.sin);
 		
 		if (iface && refresh_interface(iface)) 
 		{
@@ -1410,6 +1464,7 @@ create_interface(
 	/*
 	 * put into our interface list
 	 */
+	add_addr_to_list(&interface->sin, interface);
 	add_interface(interface);
 
 	DPRINT_INTERFACE(2, (interface, "created ", "\n"));
@@ -1581,7 +1636,7 @@ enable_multicast_if(struct interface *iface, struct sockaddr_storage *maddr)
 	case AF_INET:
 		if (setsockopt(iface->fd, IPPROTO_IP, IP_MULTICAST_IF,
 		   (char *)&(((struct sockaddr_in*)&iface->sin)->sin_addr.s_addr),
-		    sizeof(struct sockaddr_in*)) == -1) {
+		    sizeof(struct in_addr)) == -1) {
 			netsyslog(LOG_ERR,
 			"setsockopt IP_MULTICAST_IF failure: %m on socket %d, addr %s for multicast address %s",
 			iface->fd, stoa(&iface->sin), stoa(maddr));
@@ -1941,19 +1996,22 @@ io_multicast_add(
 	set_reuseaddr(1);
 	interface->bfd = INVALID_SOCKET;
 	interface->fd = open_socket(&interface->sin,
-			    INT_MULTICAST, 1, interface);
+			    INT_MULTICAST, 0, interface);
 
 	if (interface->fd != INVALID_SOCKET)
 	{
 		interface->bfd = INVALID_SOCKET;
 		interface->ignore_packets = ISC_FALSE;
+		interface->flags |= INT_MCASTIF;
 		
 		(void) strncpy(interface->name, "multicast",
 			sizeof(interface->name));
 		((struct sockaddr_in*)&interface->mask)->sin_addr.s_addr =
 						htonl(~(u_int32)0);
 		DPRINT_INTERFACE(2, (interface, "multicast add ", "\n"));
+		add_addr_to_list(&interface->sin, interface);
 		add_interface(interface);
+		list_if_listening(interface, NTP_PORT);
 	}
 	else
 	{
@@ -2194,6 +2252,7 @@ open_socket(
 		    errval == WSAEPFNOSUPPORT)
 #endif
 			return (INVALID_SOCKET);
+		msyslog(LOG_ERR, "unexpected error code %d (not PROTONOSUPPORT|AFNOSUPPORT|FPNOSUPPORT) - exiting", errval);
 		exit(1);
 		/*NOTREACHED*/
 	}
@@ -2212,7 +2271,7 @@ open_socket(
 
 	/*
 	 * set SO_REUSEADDR since we will be binding the same port
-	 * number on each interface
+	 * number on each interface according to flag
 	 */
 	if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
 		       turn_off_reuse ? (char *)&off : (char *)&on, sizeof(on)))
@@ -2263,34 +2322,34 @@ open_socket(
 #endif /* IPV6_BINDV6ONLY */
 	}
 
+#ifdef OS_NEEDS_REUSEADDR_FOR_IFADDRBIND
+	/*
+	 * some OSes don't allow bindinf to more specific
+	 * addresses if a wildcard address already bound
+	 * to the port and SO_REUSEADDR is not set
+	 */
+	if (!is_wildcard_addr(addr)) {
+		set_wildcard_reuse(addr->ss_family, 1);
+	}
+#endif
+
 	/*
 	 * bind the local address.
 	 */
-	if (bind(fd, (struct sockaddr *)addr, SOCKLEN(addr)) < 0) {
-		char buff[160];
+	errval = bind(fd, (struct sockaddr *)addr, SOCKLEN(addr));
 
-		if(addr->ss_family == AF_INET)
-			sprintf(buff,
-				"bind() fd %d, family %d, port %d, addr %s, in_classd=%d flags=0x%x fails: %%m",
-				fd, addr->ss_family, (int)ntohs(((struct sockaddr_in*)addr)->sin_port),
-				stoa(addr),
-				IN_CLASSD(ntohl(((struct sockaddr_in*)addr)->sin_addr.s_addr)), flags);
-#ifdef INCLUDE_IPV6_SUPPORT
-		else if(addr->ss_family == AF_INET6)
-		                sprintf(buff,
-                                "bind() fd %d, family %d, port %d, scope %d, addr %s, in6_is_addr_multicast=%d flags=%d fails: %%m",
-                                fd, addr->ss_family, (int)ntohs(((struct sockaddr_in6*)addr)->sin6_port),
-# ifdef ISC_PLATFORM_HAVESCOPEID
-                                ((struct sockaddr_in6*)addr)->sin6_scope_id
-# else
-                                -1
-# endif
-				, stoa(addr),
-                                IN6_IS_ADDR_MULTICAST(&((struct sockaddr_in6*)addr)->sin6_addr), flags);
+#ifdef OS_NEEDS_REUSEADDR_FOR_IFADDRBIND
+	/*
+	 * some OSes don't allow binding to more specific
+	 * addresses if a wildcard address already bound
+	 * to the port and REUSE_ADDR is not set
+	 */
+	if (!is_wildcard_addr(addr)) {
+		set_wildcard_reuse(addr->ss_family, 0);
+	}
 #endif
-		else
-			return INVALID_SOCKET;
 
+	if (errval < 0) {
 		/*
 		 * Don't log this under all conditions
 		 */
@@ -2298,12 +2357,31 @@ open_socket(
 #ifdef DEBUG
 		    || debug > 1
 #endif
-		   )
-			netsyslog(LOG_ERR, buff);
+			) {
+			if (addr->ss_family == AF_INET)
+				netsyslog(LOG_ERR,
+					  "bind() fd %d, family %d, port %d, addr %s, in_classd=%d flags=0x%x fails: %m",
+					  fd, addr->ss_family, (int)ntohs(((struct sockaddr_in*)addr)->sin_port),
+					  stoa(addr),
+					  IN_CLASSD(ntohl(((struct sockaddr_in*)addr)->sin_addr.s_addr)), flags);
+#ifdef INCLUDE_IPV6_SUPPORT
+			else if (addr->ss_family == AF_INET6)
+		                netsyslog(LOG_ERR,
+					  "bind() fd %d, family %d, port %d, scope %d, addr %s, in6_is_addr_multicast=%d flags=%d fails: %m",
+					  fd, addr->ss_family, (int)ntohs(((struct sockaddr_in6*)addr)->sin6_port),
+# ifdef ISC_PLATFORM_HAVESCOPEID
+					  ((struct sockaddr_in6*)addr)->sin6_scope_id
+# else
+					  -1
+# endif
+					  , stoa(addr),
+					  IN6_IS_ADDR_MULTICAST(&((struct sockaddr_in6*)addr)->sin6_addr), flags);
+#endif
+		}
 
 		closesocket(fd);
-
-		return (INVALID_SOCKET);
+		
+		return INVALID_SOCKET;
 	}
 
 #ifdef HAVE_TIMESTAMP
@@ -2338,22 +2416,6 @@ open_socket(
 
 	add_fd_to_list(fd, FD_TYPE_SOCKET);
 
-	add_addr_to_list(addr, interf);
-
-	/*
-	 *	Turn off the SO_REUSEADDR socket option.  It apparently
-	 *	causes heartburn on systems with multicast IP installed.
-	 *	On normal systems it only gets looked at when the address
-	 *	is being bound anyway..
-	 */
-	if (turn_off_reuse)
-	    if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
-			   (char *)&off, sizeof(off)))
-	    {
-		    netsyslog(LOG_ERR, "setsockopt SO_REUSEADDR off fails on address %s: %m",
-			    stoa(addr));
-	    }
-
 #if !defined(SYS_WINNT) && !defined(VMS)
 	DPRINTF(4, ("flags for fd %d: 0x%x\n", fd,
 		    fcntl(fd, F_GETFL, 0)));
@@ -2748,7 +2810,7 @@ static l_fp
 static inline int
 read_network_packet(SOCKET fd, struct interface *itf, l_fp ts)
 {
-	size_t fromlen;
+	GETSOCKNAME_SOCKLEN_TYPE fromlen;
 	int buflen;
 	register struct recvbuf *rb;
 #ifdef HAVE_TIMESTAMP
@@ -2991,14 +3053,16 @@ input_handler(
 	/*
 	 * scan list of asyncio readers - currently only used for routing sockets
 	 */
-	for (asyncio_reader = ISC_LIST_TAIL(asyncio_reader_list);
-	     asyncio_reader != NULL;
-	     asyncio_reader = ISC_LIST_PREV(asyncio_reader, link))
+	asyncio_reader = ISC_LIST_TAIL(asyncio_reader_list);
+
+	while (asyncio_reader != NULL)
 	{
+	        struct asyncio_reader *next = ISC_LIST_PREV(asyncio_reader, link);
 		if (FD_ISSET(asyncio_reader->fd, &fds)) {
 			++select_count;
 			asyncio_reader->receiver(asyncio_reader);
 		}
+		asyncio_reader = next;
 	}
 #endif /* HAS_ROUTING_SOCKET */
 	
@@ -3078,16 +3142,15 @@ findinterface(
  * logic in ntpd which would be a silly and really unportable thing to do.
  *
  */
-struct interface *
+static struct interface *
 findlocalinterface(
 	struct sockaddr_storage *addr
 	)
 {
 	SOCKET s;
 	int rtn;
-	struct interface *interface;
 	struct sockaddr_storage saddr;
-	size_t saddrlen = SOCKLEN(addr);
+	GETSOCKNAME_SOCKLEN_TYPE saddrlen = SOCKLEN(addr);
 
 	DPRINTF(4, ("Finding interface for addr %s in list of addresses\n",
 		    stoa(addr));)
@@ -3134,29 +3197,25 @@ findlocalinterface(
 
 	DPRINTF(4, ("findlocalinterface: kernel maps %s to %s\n", stoa(addr), stoa(&saddr)));
 	
-	for (interface = ISC_LIST_HEAD(inter_list);
-	     interface != NULL;
-	     interface = ISC_LIST_NEXT(interface, link)) 
-	{
-		if (interface->flags & INT_WILDCARD)
-			continue;
-	  
+	return getinterface(&saddr);
+}
+
+/*
+ * fetch an interface structure the matches the
+ * address is not a wildcard/mcast or ignoring interface
+ */
+static struct interface *
+getinterface(struct sockaddr_storage *addr)
+{
+	struct interface *interface = find_addr_in_list(addr);
+
+	if (interface != NULL) {
+		if (interface->flags & (INT_WILDCARD|INT_MCASTIF))
+			return NULL;
+		
 		/* Don't both with ignore interfaces */
 		if (interface->ignore_packets == ISC_TRUE)
-			continue;
-
-		/*
-		 * First look if is the the correct family
-		 */
-		if(interface->sin.ss_family != saddr.ss_family)
-			continue;
-		/*
-		 * We match the unicast address only.
-		 */
-		if (SOCKCMP(&interface->sin, &saddr))
-		{
-			break;
-		}
+			return NULL;
 	}
 
 	return interface;
@@ -3256,8 +3315,15 @@ findbcastinter(
 	DPRINTF(4, ("Finding broadcast interface for addr %s in list of addresses\n",
 		    stoa(addr)));
 
+#if 0 /* currently disabled - see comment inside */
 	interface = find_flagged_addr_in_list(addr, INT_BCASTOPEN|INT_MCASTOPEN);
 	
+	/*
+	 * Do nothing right now
+	 * Eventually we will find the interface this
+	 * way, but until it works properly we just see
+	 * which one we got
+	 */
 #ifdef DEBUG
 	if (interface) {
 		DPRINTF(4, ("Found bcastinter index #%d %s\n", interface->ifnum, interface->name));
@@ -3265,29 +3331,23 @@ findbcastinter(
 		DPRINTF(4, ("No bcast interface found for %s\n", stoa(addr)));
 	}
 #endif
-	/*
-	 * Do nothing right now
-	 * Eventually we will find the interface this
-	 * way, but until it works properly we just see
-	 * which one we got
-	 */
-/*	if (interface != NULL)
+	if (interface != NULL)
 	{
-		return (interface);
+		return interface;
 	}
-*/
+#endif
 
 	for (interface = ISC_LIST_HEAD(inter_list);
 	     interface != NULL;
 	     interface = ISC_LIST_NEXT(interface, link)) 
-	  {
+	{
 	        if (interface->flags & INT_WILDCARD)
 		        continue;
-	  
+		
 		/* Don't bother with ignored interfaces */
 		if (interface->ignore_packets == ISC_TRUE)
 			continue;
-
+		
 		/*
 		 * First look if this is the correct family
 		 */
@@ -3298,22 +3358,31 @@ findbcastinter(
 		if (interface->flags & INT_LOOPBACK)
 			continue;
 
+#if 0 /* XXX - disable null effect code - kd */
 		/* for IPv6 multicast check the address for linklocal */
 #ifdef INCLUDE_IPV6_SUPPORT
 		if (interface->sin.ss_family == AF_INET6 &&
-		   (IN6_IS_ADDR_LINKLOCAL(&((struct sockaddr_in6*)&interface->sin)->sin6_addr)))
+		    (IN6_IS_ADDR_LINKLOCAL(&((struct sockaddr_in6*)&interface->sin)->sin6_addr)))
 		{
 /*			continue; */
 		}
+#endif
 #endif
 		/*
 		 * If we are looking to match a multicast address grab it.
-		 * We must not do this before we have eliminated any linklocal
-		 * addresses
 		 */
 		if (addr_ismulticast(addr) == ISC_TRUE && interface->flags & INT_MULTICAST)
 		{
-			return (interface);
+#ifdef INCLUDE_IPV6_SUPPORT
+			if(addr->ss_family == AF_INET6) {
+				/* Only use link-local address for link-scope mcast */
+				if(IN6_IS_ADDR_MC_LINKLOCAL(&((struct sockaddr_in6*)addr)->sin6_addr) &&
+				  !IN6_IS_ADDR_LINKLOCAL(&((struct sockaddr_in6*)&interface->sin)->sin6_addr)) {
+					continue;
+				}
+			}
+#endif
+			break;
 		}
 
 		/*
@@ -3323,24 +3392,33 @@ findbcastinter(
 		 * Sloppy.
 		 */
 		if(addr->ss_family == AF_INET) {
-			if (SOCKCMP(&interface->bcast, addr))
-				return interface;
+			if (SOCKCMP(&interface->bcast, addr)) {
+				break;
+			}
 			if ((NSRCADR(&interface->sin) &
-				NSRCADR(&interface->mask)) == (NSRCADR(addr) &
-			    	NSRCADR(&interface->mask)))
-				return interface;
+			     NSRCADR(&interface->mask)) == (NSRCADR(addr) &
+							    NSRCADR(&interface->mask)))
+				break;
 		}
 #ifdef INCLUDE_IPV6_SUPPORT
 		else if(addr->ss_family == AF_INET6) {
-			if (SOCKCMP(&interface->bcast, addr))
-				return interface;
-			if (SOCKCMP(netof(&interface->sin), netof(addr)))
-				return interface;
+			if (SOCKCMP(&interface->bcast, addr)) {
+				break;
+			}
+			if (SOCKCMP(netof(&interface->sin), netof(addr))) {
+				break;
+			}
 		}
 #endif
 	}
 #endif /* SIOCGIFCONF */
- 	return ANY_INTERFACE_CHOOSE(addr);
+	if (interface == NULL) {
+		DPRINTF(4, ("No bcast interface found for %s\n", stoa(addr)));
+		return ANY_INTERFACE_CHOOSE(addr);
+	} else {
+		DPRINTF(4, ("Found bcastinter index #%d %s\n", interface->ifnum, interface->name));
+		return interface;
+	}
 }
 
 
@@ -3482,7 +3560,7 @@ kill_asyncio(int startfd)
 /*
  * Add and delete functions for the list of open sockets
  */
-void
+static void
 add_fd_to_list(SOCKET fd, enum desc_type type) {
 	vsock_t *lsock = (vsock_t *)malloc(sizeof(vsock_t));
 	lsock->fd = fd;
@@ -3502,7 +3580,7 @@ add_fd_to_list(SOCKET fd, enum desc_type type) {
 #endif
 }
 
-void
+static void
 close_and_delete_fd_from_list(SOCKET fd) {
 
 	vsock_t *next;
@@ -3552,19 +3630,29 @@ close_and_delete_fd_from_list(SOCKET fd) {
 	}
 }
 
-void
+static void
 add_addr_to_list(struct sockaddr_storage *addr, struct interface *interface){
-	remaddr_t *laddr = (remaddr_t *)emalloc(sizeof(remaddr_t));
-	memcpy(&laddr->addr, addr, sizeof(struct sockaddr_storage));
-	laddr->interface = interface;
-
-	ISC_LIST_APPEND(remoteaddr_list, laddr, link);
-
-	DPRINTF(4, ("Added addr %s to list of addresses\n",
-		    stoa(addr)));
+#ifdef DEBUG
+	if (find_addr_in_list(addr) == NULL) {
+#endif
+		/* not there yet - add to list */
+		remaddr_t *laddr = (remaddr_t *)emalloc(sizeof(remaddr_t));
+		memcpy(&laddr->addr, addr, sizeof(struct sockaddr_storage));
+		laddr->interface = interface;
+		
+		ISC_LIST_APPEND(remoteaddr_list, laddr, link);
+		
+		DPRINTF(4, ("Added addr %s to list of addresses\n",
+			    stoa(addr)));
+#ifdef DEBUG
+	} else {
+		DPRINTF(4, ("WARNING: Attempt to add duplicate addr %s to address list\n",
+			    stoa(addr)));
+	}
+#endif
 }
 
-void
+static void
 delete_addr_from_list(struct sockaddr_storage *addr) {
 
 	remaddr_t *next;
@@ -3576,38 +3664,39 @@ delete_addr_from_list(struct sockaddr_storage *addr) {
 			ISC_LIST_DEQUEUE_TYPE(remoteaddr_list, laddr, link, remaddr_t);
 			free(laddr);
 			break;
+			DPRINTF(4, ("Deleted addr %s from list of addresses\n",
+				    stoa(addr)));
 		}
 		else
 			laddr = next;
 	}
-	DPRINTF(4, ("Deleted addr %s from list of addresses\n",
-		    stoa(addr)));
 }
 
-struct interface *
+static struct interface *
 find_addr_in_list(struct sockaddr_storage *addr) {
 
 	remaddr_t *next;
 	remaddr_t *laddr = ISC_LIST_HEAD(remoteaddr_list);
-	DPRINTF(4, ("Finding addr %s in list of addresses\n",
+	DPRINTF(4, ("Searching for addr %s in list of addresses - ",
 		    stoa(addr)));
 
 	while(laddr != NULL) {
 		next = ISC_LIST_NEXT(laddr, link);
 		if(SOCKCMP(&laddr->addr, addr)) {
+			DPRINTF(4, ("FOUND\n"));
 			return laddr->interface;
-			break;
 		}
 		else
 			laddr = next;
 	}
+	DPRINTF(4, ("NOT FOUND\n"));
 	return NULL; /* Not found */
 }
 
 /*
  * Find the given address with the associated flag in the list
  */
-struct interface *
+static struct interface *
 find_flagged_addr_in_list(struct sockaddr_storage *addr, int flag) {
 
 	remaddr_t *next;
@@ -3630,6 +3719,10 @@ find_flagged_addr_in_list(struct sockaddr_storage *addr, int flag) {
 #ifdef HAS_ROUTING_SOCKET
 #include <net/route.h>
 
+#ifndef UPDATE_GRACE
+#define UPDATE_GRACE	2	/* wait UPDATE_GRACE seconds before scanning */
+#endif
+
 static void
 process_routing_msgs(struct asyncio_reader *reader)
 {
@@ -3638,6 +3731,16 @@ process_routing_msgs(struct asyncio_reader *reader)
 
 	int cnt;
 	
+	if (disable_dynamic_updates) {
+		/*
+		 * discard ourselves if we are not need any more
+		 * usually happens when running unprivileged
+		 */
+		remove_asyncio_reader(reader);
+		delete_asyncio_reader(reader);
+		return;
+	}
+
 	cnt = read(reader->fd, buffer, sizeof(buffer));
 	
 	if (cnt < 0) {
@@ -3694,7 +3797,7 @@ process_routing_msgs(struct asyncio_reader *reader)
 			 * we are keen on new and deleted addresses and if an interface goes up and down or routing changes
 			 */
 			DPRINTF(3, ("routing message op = %d: scheduling interface update\n", rtm->rtm_type));
-			timer_interfacetimeout(current_time);
+			timer_interfacetimeout(current_time + UPDATE_GRACE);
 			break;
 		default:
 			/*
@@ -3728,7 +3831,7 @@ init_async_notifications()
 		reader->fd = fd;
 		reader->receiver = process_routing_msgs;
 		
-		add_asyncio_reader(reader, FD_TYPE_FILE);
+		add_asyncio_reader(reader, FD_TYPE_SOCKET);
 		msyslog(LOG_INFO, "Listening on routing socket on fd #%d for interface updates", fd);
 	} else {
 		msyslog(LOG_ERR, "unable to open routing socket (%m) - using polled interface update");
