@@ -129,10 +129,12 @@ static double clock_offset;	/* offset (s) */
 double	clock_jitter;		/* offset jitter (s) */
 double	drift_comp;		/* frequency (s/s) */
 double	clock_stability;	/* frequency stability (wander) (s/s) */
+double	clock_codec;		/* audio codec frequency (sambles/s) */
 u_long	sys_clocktime;		/* last system clock update */
 u_long	pps_control;		/* last pps update */
-u_long	sys_tai;		/* UTC offset from TAI (s) */
-static void rstclock P((int, u_long, double)); /* transition function */
+u_int	sys_tai;		/* TAI offset from UTC (s) */
+u_long	sys_leapin;		/* seconds at next leap (s) */
+static void rstclock (int, u_long, double); /* transition function */
 
 #ifdef KERNEL_PLL
 struct timex ntv;		/* kernel API parameters */
@@ -142,9 +144,9 @@ int	pll_status;		/* status bits for kernel pll */
 /*
  * Clock state machine control flags
  */
-int	ntp_enable;		/* clock discipline enabled */
+int	ntp_enable = 1;		/* clock discipline enabled */
 int	pll_control;		/* kernel support available */
-int	kern_enable;		/* kernel support enabled */
+int	kern_enable = 1;	/* kernel support enabled */
 int	pps_enable;		/* kernel PPS discipline enabled */
 int	ext_enable;		/* external clock enabled */
 int	pps_stratum;		/* pps stratum */
@@ -172,7 +174,7 @@ static double sys_mindly;	/* huff-n'-puff filter min delay */
 #define MOD_BITS (MOD_OFFSET | MOD_MAXERROR | MOD_ESTERROR | \
     MOD_STATUS | MOD_TIMECONST)
 #ifdef SIGSYS
-static void pll_trap P((int));	/* configuration trap */
+static void pll_trap (int);	/* configuration trap */
 static struct sigaction sigsys;	/* current sigaction status */
 static struct sigaction newsigsys; /* new sigaction status */
 static sigjmp_buf env;		/* environment var. for pll_trap() */
@@ -218,12 +220,6 @@ local_clock(
 	double	plladj;		/* PLL frequency adjustment (ppm) */
 	double	clock_frequency; /* clock frequency adjustment (ppm) */
 	double	dtemp, etemp;	/* double temps */
-#ifdef OPENSSL
-	u_int32 *tpt;
-	int	i;
-	u_int	len;
-	long	togo;
-#endif /* OPENSSL */
 
 	/*
 	 * If the loop is opened or the NIST LOCKCLOCK is in use,
@@ -332,7 +328,8 @@ local_clock(
 	 * these actions interact with the command line options.
 	 *
 	 * Note the system poll is set to minpoll only if the clock is
-	 * stepped. 
+	 * stepped. Note also the kernel is disabled if step is
+	 * disabled or greater than 0.5 s. 
 	 */
 	clock_frequency = flladj = plladj = 0;
 	mu = peer->epoch - sys_clocktime;
@@ -404,7 +401,6 @@ local_clock(
 			reinit_timer();
 			tc_counter = 0;
 			sys_poll = NTP_MINPOLL;
-			sys_tai = 0;
 			clock_jitter = LOGTOD(sys_precision);
 			rval = 2;
 			if (state == S_NSET) {
@@ -499,35 +495,6 @@ local_clock(
 		}
 	}
 
-#ifdef OPENSSL
-	/*
-	 * Scan the loopsecond table to determine the TAI offset. If
-	 * there is a scheduled leap in future, set the leap warning,
-	 * but only if less than 30 days before the leap.
-	 */
-	tpt = (u_int32 *)tai_leap.ptr;
-	len = ntohl(tai_leap.vallen) / sizeof(u_int32);
-	if (tpt != NULL) {
-		for (i = 0; i < len; i++) {
-			togo = (long)ntohl(tpt[i]) -
-			    (long)peer->rec.l_ui;
-			if (togo > 0) {
-				if (togo < CLOCK_JUNE)
-					leap_next |= LEAP_ADDSECOND;
-				break;
-			}
-		}
-#if defined(STA_NANO) && NTP_API == 4
-		if (pll_control && kern_enable && sys_tai == 0) {
-			memset(&ntv, 0, sizeof(ntv));
-			ntv.modes = MOD_TAI;
-			ntv.constant = i + TAI_1972 - 1;
-			ntp_adjtime(&ntv);
-		}
-#endif /* STA_NANO */
-		sys_tai = i + TAI_1972 - 1;
-	}
-#endif /* OPENSSL */
 #ifdef KERNEL_PLL
 	/*
 	 * This code segment works when clock adjustments are made using
@@ -544,7 +511,7 @@ local_clock(
 	 * lead to overflow problems. This might occur if some misguided
 	 * lad set the step threshold to something ridiculous.
 	 */
-	if (pll_control && kern_enable && clock_max < 0.5) {
+	if (pll_control && kern_enable) {
 
 		/*
 		 * We initialize the structure for the ntp_adjtime()
@@ -616,16 +583,6 @@ local_clock(
 				}
 			}
 
-			/*
-			 * Switch to FLL mode if the poll interval is
-			 * greater than MAXDPOLL, so that the kernel
-			 * loop behaves as the daemon loop; viz.,
-			 * selects the FLL when necessary, etc. For
-			 * legacy only.
-			 */
-			if (sys_poll > NTP_MAXDPOLL)
-				ntv.status |= STA_FLL;
-
 			/*
 			 * If the PPS signal is up and enabled, light
 			 * the frequency bit. If the PPS driver is
@@ -649,17 +606,15 @@ local_clock(
 		 * frequency and pretend we did it here.
 		 */
 		if (ntp_adjtime(&ntv) == TIME_ERROR) {
-			if (ntv.status != pll_status)
-				NLOG(NLOG_SYNCEVENT | NLOG_SYSEVENT)
-				    msyslog(LOG_NOTICE,
-				    "kernel time sync disabled %04x",
-				    ntv.status);
+			NLOG(NLOG_SYNCEVENT | NLOG_SYSEVENT)
+			    msyslog(LOG_NOTICE,
+			    "kernel time sync error %04x", ntv.status);
 			ntv.status &= ~(STA_PPSFREQ | STA_PPSTIME);
 		} else {
-			if (ntv.status != pll_status)
+			if ((ntv.status ^ pll_status) & ~STA_FLL)
 				NLOG(NLOG_SYNCEVENT | NLOG_SYSEVENT)
 				    msyslog(LOG_NOTICE,
-				    "kernel time sync enabled %04x",
+				    "kernel time sync status change %04x",
 				    ntv.status);
 		}
 		pll_status = ntv.status;
@@ -796,7 +751,7 @@ adj_host_clock(
 	 * get out of Dodge quick.
 	 */
 	if (!ntp_enable || mode_ntpdate || (pll_control &&
-	    kern_enable && clock_max < 0.5))
+	    kern_enable))
 		return;
 
 	/*
@@ -895,15 +850,8 @@ loop_config(
 		 * behind. While at it, ask to set nanosecond mode. If
 		 * the kernel agrees, rejoice; othewise, it does only
 		 * microseconds.
-		 *
-		 * Call out the safety patrol. If ntpdate mode or if the
-		 * step threshold has been increased by the -x option or
-		 * tinker command, kernel discipline is unsafe, so don't
-		 * do any of this stuff. Otherwise, initialize the
-		 * kernel to appear unsynchronized until the first
-		 * update is received.
 		 */
-		if (mode_ntpdate || clock_max > CLOCK_MAX)
+		if (mode_ntpdate)
 			break;
 
 		pll_control = 1;
@@ -986,7 +934,7 @@ loop_config(
 		 */
 		if (pll_control && kern_enable) {
 			memset((char *)&ntv, 0, sizeof(ntv));
-			ntv.modes = MOD_FREQUENCY;
+			ntv.modes = MOD_OFFSET | MOD_FREQUENCY;
 			ntv.freq = (int32)(drift_comp * 65536e6);
 			ntp_adjtime(&ntv);
 		}
@@ -1000,7 +948,7 @@ loop_config(
 		/* Completely turn off the kernel time adjustments. */
 		if (pll_control) {
 			memset((char *)&ntv, 0, sizeof(ntv));
-			ntv.modes = MOD_BITS | MOD_FREQUENCY;
+			ntv.modes = MOD_BITS | MOD_OFFSET | MOD_FREQUENCY;
 			ntv.status = STA_UNSYNC;
 			ntp_adjtime(&ntv);
 			NLOG(NLOG_SYNCEVENT | NLOG_SYSEVENT)
@@ -1017,6 +965,8 @@ loop_config(
 	 */
 	case LOOP_MAX:			/* step threshold */
 		clock_max = freq;
+		if (clock_max == 0 || clock_max > 0.5)
+		kern_enable = 0;
 		break;
 
 	case LOOP_PANIC:		/* panic threshold */
@@ -1050,6 +1000,21 @@ loop_config(
 		drift_comp = freq / 1e6;
 		rstclock(S_FSET, 0, 0);
 		break;
+
+	case LOOP_CODEC:		/* audio codec frequency */
+		clock_codec = freq;
+		break;
+
+	case LOOP_LEAP:			/* arm leap insert */
+#if defined(STA_NANO) && NTP_API == 4
+		if (pll_control && kern_enable) {
+			memset(&ntv, 0, sizeof(ntv));
+			ntv.modes = MOD_TAI;
+			ntv.constant = sys_tai - 1;
+			ntp_adjtime(&ntv);
+		}
+#endif /* STA_NANO */
+		break;
 	}
 }
 
