@@ -103,8 +103,8 @@ int peer_associations;			/* mobilized associations */
 int peer_preempt;			/* preemptable associations */
 static struct peer init_peer_alloc[INIT_PEER_ALLOC]; /* init alloc */
 
-static void	    getmorepeermem	 P((void));
-static struct interface *select_peerinterface P((struct peer *, struct sockaddr_storage *, struct interface *, u_char));
+static void	    getmorepeermem	 (void);
+static struct interface *select_peerinterface (struct peer *, struct sockaddr_storage *, struct interface *, u_char);
 
 /*
  * init_peer - initialize peer data structures and counters
@@ -530,8 +530,6 @@ void
 set_peerdstadr(struct peer *peer, struct interface *interface)
 {
 	if (peer->dstadr != interface) {
-		struct interface *prev_dstadr = peer->dstadr;
-
 		if (peer->dstadr != NULL)
 		{
 			peer->dstadr->peercnt--;
@@ -545,16 +543,6 @@ set_peerdstadr(struct peer *peer, struct interface *interface)
 
 		peer->dstadr = interface;
 
-		if (prev_dstadr != NULL) {
-			/*
-			 * reset crypto information if we change from an
-			 * active interface
-			 * all other crypto updates are handled by the crypto
-			 * machinery
-			 */
-			peer_crypto_clear(peer);
-		}
-
 		if (peer->dstadr != NULL)
 		{
 			ISC_LIST_APPEND(peer->dstadr->peers, peer, ilink);
@@ -569,7 +557,7 @@ set_peerdstadr(struct peer *peer, struct interface *interface)
 static void
 peer_refresh_interface(struct peer *peer)
 {
-	struct interface *niface;
+	struct interface *niface, *piface;
 
 	niface = select_peerinterface(peer, &peer->srcadr, NULL, peer->cast_flags);
 
@@ -607,20 +595,31 @@ peer_refresh_interface(struct peer *peer)
 	}
 #endif
 
+	piface = peer->dstadr;
+
 	set_peerdstadr(peer, niface);
 
-	/*
-	 * Broadcast needs the socket enabled for broadcast
-	 */
-	if (peer->cast_flags & MDF_BCAST && peer->dstadr) {
-		enable_broadcast(peer->dstadr, &peer->srcadr);
-	}
+	if (peer->dstadr) {
+                /*
+                 * clear crypto if we change the local address
+                 */
+                if (peer->dstadr != piface) {
+                        peer_crypto_clear(peer);
+                }
 
-	/*
-	 * Multicast needs the socket interface enabled for multicast
-	 */
-	if (peer->cast_flags & MDF_MCAST) {
-		enable_multicast_if(peer->dstadr, &peer->srcadr);
+		/*
+	 	 * Broadcast needs the socket enabled for broadcast
+	 	 */
+		if (peer->cast_flags & MDF_BCAST) {
+			enable_broadcast(peer->dstadr, &peer->srcadr);
+		}
+
+		/*
+	 	 * Multicast needs the socket interface enabled for multicast
+	 	 */
+		if (peer->cast_flags & MDF_MCAST) {
+			enable_multicast_if(peer->dstadr, &peer->srcadr);
+		}
 	}
 }
 
@@ -670,10 +669,10 @@ select_peerinterface(struct peer *peer, struct sockaddr_storage *srcadr, struct
 #ifdef DEBUG
 			if (debug > 3) {
 				if (interface != NULL)
-					printf("Found broadcast interface address %s, for address %s\n",
+					printf("Found *-cast interface address %s, for address %s\n",
 					       stoa(&(interface)->sin), stoa(srcadr));
 				else
-					printf("No broadcast local address found for address %s\n",
+					printf("No *-cast local address found for address %s\n",
 					       stoa(srcadr));
 			}
 #endif
