@@ -58,34 +58,26 @@
 #define	INITRESLIST	10
 #define	INCRESLIST	5
 
-#define RES_AVG		8.	/* interpacket averaging factor */
-
 /*
  * The restriction list
  */
 struct restrictlist *restrictlist;
 struct restrictlist6 *restrictlist6;
-static	int restrictcount;	/* count of entries in the res list */
-static	int restrictcount6;	/* count of entries in the res list 2*/
+static int restrictcount;	/* count of entries in the res list */
+static int restrictcount6;	/* count of entries in the res list 2*/
 
 /*
  * The free list and associated counters.  Also some uninteresting
  * stat counters.
  */
-static	struct restrictlist *resfree;
-static	struct restrictlist6 *resfree6;
-static	int numresfree;		/* number of structures on free list */
-static	int numresfree6;	/* number of structures on free list 2 */
+static struct restrictlist *resfree;
+static struct restrictlist6 *resfree6;
+static int numresfree;		/* number of struct on free list */
+static int numresfree6;	/* number of struct on free list 2 */
 
-static	u_long res_calls;
-static	u_long res_found;
-static	u_long res_not_found;
-
-/*
- * Parameters of the RES_LIMITED restriction option.
- */
-u_long res_avg_interval = 5;	/* min average interpacket interval */
-u_long res_min_interval = 1;	/* min interpacket interval */
+static u_long res_calls;
+static u_long res_found;
+static u_long res_not_found;
 
 /*
  * Count number of restriction entries referring to RES_LIMITED controls
@@ -101,6 +93,7 @@ static	u_long res_limited_refcnt6;
 static	struct restrictlist resinit[INITRESLIST];
 static	struct restrictlist6 resinit6[INITRESLIST];
 
+
 /*
  * init_restrict - initialize the restriction data structures
  */
@@ -172,9 +165,11 @@ restrictions(
 	int	isntpport;
 
 	res_calls++;
+	/* IPv4 source address */
 	if (srcadr->ss_family == AF_INET) {
+
 		/*
-		 * We need the host address in host order.  Also need to
+		 * We need the host address in host order. Also need to
 		 * know whether this is from the ntp port or not.
 		 */
 		hostaddr = SRCADR(srcadr);
@@ -183,7 +178,7 @@ restrictions(
 		/*
 		 * Ignore any packets with a multicast source address
 		 * (this should be done early in the receive process,
-		 * later!)
+		 * not later!)
 		 */
 		if (IN_CLASSD(SRCADR(srcadr)))
 			return (int)RES_IGNORE;
@@ -211,9 +206,10 @@ restrictions(
 
 	/* IPv6 source address */
 	if (srcadr->ss_family == AF_INET6) {
+
 		/*
-		 * Need to know whether this is from the ntp port or
-		 * not.
+		 * We need the host address in host order. Also need to
+		 * know whether this is from the ntp port or not.
 		 */
 		hostaddr6 = GET_INADDR6(*srcadr);
 		isntpport = (ntohs((
@@ -223,7 +219,7 @@ restrictions(
 		/*
 		 * Ignore any packets with a multicast source address
 		 * (this should be done early in the receive process,
-		 * later!)
+		 * not later!)
 		 */
 		if (IN6_IS_ADDR_MULTICAST(&hostaddr6))
 			return (int)RES_IGNORE;
@@ -253,35 +249,6 @@ restrictions(
 			res_found++;
 		flags = match6->flags;
 	}
-
-	/*
-	 * The following implements a generalized call gap facility.
-	 * Douse the RES_LIMITED bit only if the interval since the last
-	 * packet is greater than res_min_interval and the average is
-	 * greater thatn res_avg_interval.
-	 */
-	if (mon_enabled == MON_OFF) {
-		flags &= ~RES_LIMITED;
-	} else {
-		struct mon_data *md;
-
-		/*
-		 * At this poin the most recent arrival is first in the
-		 * MRU list. Let the first 10 packets in for free until
-		 * the average stabilizes.
-		 */
-		md = mon_mru_list.mru_next;
-		if (md->avg_interval == 0)
-			md->avg_interval = md->drop_count;
-		else
-			md->avg_interval += (md->drop_count -
-			    md->avg_interval) / RES_AVG;
-		if (md->count < 10 || (md->drop_count >
-		    res_min_interval && md->avg_interval >
-		    res_avg_interval))
-			flags &= ~RES_LIMITED;
-		md->drop_count = flags;
-	}
 	return (flags);
 }
 
@@ -311,6 +278,12 @@ hack_restrict(
 	memset(&mask6, 0, sizeof(struct in6_addr)); 
 
 	if (resaddr->ss_family == AF_INET) {
+#ifdef DEBUG
+        if (debug)
+		printf("restrict: addr %08x mask %08x mflags %08x flags %08x\n",
+		    SRCADR(resaddr), SRCADR(resmask), mflags, flags);
+#endif
+
 		/*
 		 * Get address and mask in host byte order
 		 */
@@ -372,6 +345,7 @@ hack_restrict(
 				if (addr_cmp > 0) {
 					rl6 = 0;
 					break;
+
 				} else if (addr_cmp == 0) {
 					mask_cmp = memcmp(&rl6->mask6,
 					    &mask6, sizeof(mask6));
@@ -404,13 +378,13 @@ hack_restrict(
 	 * points to the entry prior to where this one should go in
 	 * the sort.
 	 */
-
 	/*
 	 * Switch based on operation
 	 */
 	if (resaddr->ss_family == AF_INET) {
 		switch (op) {
 		case RESTRICT_FLAGS:
+
 			/*
 			 * Here we add bits to the flags. If this is a
 			 * new restriction add it.
@@ -424,22 +398,20 @@ hack_restrict(
 					memset((char *)rl, 0,
 					    INCRESLIST * sizeof(struct
 					    restrictlist));
-					for (i = 0; i < INCRESLIST; i++) {
+					for (i = 0; i < INCRESLIST;
+					    i++) {
 						rl->next = resfree;
 						resfree = rl;
 						rl++;
 					}
 					numresfree = INCRESLIST;
 				}
-
 				rl = resfree;
 				resfree = rl->next;
 				numresfree--;
-
 				rl->addr = addr;
 				rl->mask = mask;
 				rl->mflags = (u_short)mflags;
-
 				if (rlprev == NULL) {
 					rl->next = restrictlist;
 					restrictlist = rl;
@@ -458,6 +430,7 @@ hack_restrict(
 			break;
 
 		case RESTRICT_UNFLAG:
+
 			/*
 			 * Remove some bits from the flags. If we didn't
 			 * find this one, just return.
@@ -475,6 +448,7 @@ hack_restrict(
 	
 		case RESTRICT_REMOVE:
 		case RESTRICT_REMOVEIF:
+
 			/*
 			 * Remove an entry from the table entirely if we
 			 * found one. Don't remove the default entry and
@@ -482,7 +456,8 @@ hack_restrict(
 			 */
 			if (rl != 0
 			    && rl->addr != htonl(INADDR_ANY)
-			    && !(rl->mflags & RESM_INTERFACE && op != RESTRICT_REMOVEIF)) {
+			    && !(rl->mflags & RESM_INTERFACE && op !=
+			    RESTRICT_REMOVEIF)) {
 				if (rlprev != NULL) {
 					rlprev->next = rl->next;
 				} else {
@@ -509,6 +484,7 @@ hack_restrict(
 	} else if (resaddr->ss_family == AF_INET6) {
 		switch (op) {
 		case RESTRICT_FLAGS:
+
 			/*
 			 * Here we add bits to the flags. If this is a
 			 * new restriction add it.
@@ -555,6 +531,7 @@ hack_restrict(
 			break;
 
 		case RESTRICT_UNFLAG:
+
 			/*
 			 * Remove some bits from the flags. If we didn't
 			 * find this one, just return.
@@ -572,6 +549,7 @@ hack_restrict(
 
 		case RESTRICT_REMOVE:
 		case RESTRICT_REMOVEIF:
+
 			/*
 			 * Remove an entry from the table entirely if we
 			 * found one. Don't remove the default entry and
@@ -579,7 +557,8 @@ hack_restrict(
 			 */
 			if (rl6 != 0 &&
 			    !IN6_IS_ADDR_UNSPECIFIED(&rl6->addr6)
-			    && !(rl6->mflags & RESM_INTERFACE && op != RESTRICT_REMOVEIF)) {
+			    && !(rl6->mflags & RESM_INTERFACE && op !=
+			    RESTRICT_REMOVEIF)) {
 				if (rlprev6 != NULL) {
 					rlprev6->next = rl6->next;
 				} else {
