@@ -12,13 +12,18 @@
 #include "ntp_stdlib.h"
 #include <ntp_random.h>
 
+#include "ntp_syslog.h"
+#include "isc/assertions.h"
+#include "isc/error.h"
+#include "isc/strerror.h"
+#include "isc/formatcheck.h"
+
 #ifdef SIM
 # include "ntpsim.h"
-# include "ntpdsim-opts.h"
-#else
-# include "ntpd-opts.h"
 #endif
 
+#include "ntpd-opts.h"
+
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -26,30 +31,22 @@
 # include <sys/stat.h>
 #endif
 #include <stdio.h>
-#ifndef SYS_WINNT
-# if !defined(VMS)	/*wjm*/
-#  ifdef HAVE_SYS_PARAM_H
-#   include <sys/param.h>
-#  endif
-# endif /* VMS */
-# ifdef HAVE_SYS_SIGNAL_H
-#  include <sys/signal.h>
-# else
-#  include <signal.h>
+#if !defined(VMS)	/*wjm*/
+# ifdef HAVE_SYS_PARAM_H
+#  include <sys/param.h>
 # endif
-# ifdef HAVE_SYS_IOCTL_H
-#  include <sys/ioctl.h>
-# endif /* HAVE_SYS_IOCTL_H */
-# ifdef HAVE_SYS_RESOURCE_H
-#  include <sys/resource.h>
-# endif /* HAVE_SYS_RESOURCE_H */
+#endif /* VMS */
+#ifdef HAVE_SYS_SIGNAL_H
+# include <sys/signal.h>
 #else
 # include <signal.h>
-# include <process.h>
-# include <io.h>
-# include <clockstuff.h>
-#include "ntp_iocompletionport.h"
-#endif /* SYS_WINNT */
+#endif
+#ifdef HAVE_SYS_IOCTL_H
+# include <sys/ioctl.h>
+#endif /* HAVE_SYS_IOCTL_H */
+#ifdef HAVE_SYS_RESOURCE_H
+# include <sys/resource.h>
+#endif /* HAVE_SYS_RESOURCE_H */
 #if defined(HAVE_RTPRIO)
 # ifdef HAVE_SYS_RESOURCE_H
 #  include <sys/resource.h>
@@ -86,8 +83,8 @@
 # include <apollo/base.h>
 #endif /* SYS_DOMAINOS */
 
-#include "recvbuff.h"  
-#include "ntp_cmdargs.h"  
+#include "recvbuff.h"
+#include "ntp_cmdargs.h"
 
 #if 0				/* HMS: I don't think we need this. 961223 */
 #ifdef LOCK_PROCESS
@@ -127,10 +124,10 @@
  * Signals which terminate us gracefully.
  */
 #ifndef SYS_WINNT
-# define SIGDIE1 	SIGHUP
-# define SIGDIE3 	SIGQUIT
-# define SIGDIE2 	SIGINT
-# define SIGDIE4 	SIGTERM
+# define SIGDIE1	SIGHUP
+# define SIGDIE3	SIGQUIT
+# define SIGDIE2	SIGINT
+# define SIGDIE4	SIGTERM
 #endif /* SYS_WINNT */
 
 #ifdef HAVE_DNSREGISTRATION
@@ -170,9 +167,9 @@ char *group = NULL;		/* group to switch to */
 char *chrootdir = NULL;		/* directory to chroot to */
 int sw_uid;
 int sw_gid;
-char *endp;  
+char *endp;
 struct group *gr;
-struct passwd *pw; 
+struct passwd *pw;
 #endif /* HAVE_DROPROOT */
 
 /*
@@ -194,33 +191,44 @@ int was_alarmed;
 /*
  * We put this here, since the argument profile is syscall-specific
  */
-extern int syscall	P((int, ...));
+extern int syscall	(int, ...);
 #endif /* DECL_SYSCALL */
 
 
 #ifdef	SIGDIE2
-static	RETSIGTYPE	finish		P((int));
+static	RETSIGTYPE	finish		(int);
 #endif	/* SIGDIE2 */
 
 #ifdef	DEBUG
 #ifndef SYS_WINNT
-static	RETSIGTYPE	moredebug	P((int));
-static	RETSIGTYPE	lessdebug	P((int));
+static	RETSIGTYPE	moredebug	(int);
+static	RETSIGTYPE	lessdebug	(int);
 #endif
 #else /* not DEBUG */
-static	RETSIGTYPE	no_debug	P((int));
+static	RETSIGTYPE	no_debug	(int);
 #endif	/* not DEBUG */
 
-int 		ntpdmain		P((int, char **));
-static void	set_process_priority	P((void));
-static void	init_logging		P((char const *));
-static void	setup_logfile		P((void));
+int		ntpdmain		(int, char **);
+static void	set_process_priority	(void);
+void		init_logging		(char const *, int);
+void		setup_logfile		(void);
+
+static void	assertion_failed	(const char *file, int line,
+	isc_assertiontype_t type, const char *cond);
+static void	library_fatal_error	(const char *file, int line,
+	const char *format, va_list args) ISC_FORMAT_PRINTF(3, 0);
+static void	library_unexpected_error(const char *file, int line,
+	const char *format, va_list args) ISC_FORMAT_PRINTF(3, 0);
+
 
 /*
  * Initialize the logging
  */
 void
-init_logging(char const *name)
+init_logging(
+	char const *name,
+	int log_version
+	)
 {
 	const char *cp;
 
@@ -253,7 +261,8 @@ init_logging(char const *name)
 # endif /* LOG_DAEMON */
 #endif	/* !SYS_WINNT && !VMS */
 
-	NLOG(NLOG_SYSINFO) /* conditional if clause for conditional syslog */
+	if (log_version)
+	    NLOG(NLOG_SYSINFO) /* 'if' clause for syslog */
 		msyslog(LOG_NOTICE, "%s", Version);
 }
 
@@ -324,10 +333,10 @@ main(
 /*
  * OK. AIX is different than solaris in how it implements plock().
  * If you do NOT adjust the stack limit, you will get the MAXIMUM
- * stack size allocated and PINNED with you program. To check the 
- * value, use ulimit -a. 
+ * stack size allocated and PINNED with you program. To check the
+ * value, use ulimit -a.
  *
- * To fix this, we create an automatic variable and set our stack limit 
+ * To fix this, we create an automatic variable and set our stack limit
  * to that PLUS 32KB of extra space (we need some headroom).
  *
  * This subroutine gets the stack address.
@@ -371,10 +380,6 @@ set_process_priority(void)
 			priority_done);
 #endif /* DEBUG */
 
-#ifdef SYS_WINNT
-	priority_done += NT_set_process_priority();
-#endif
-
 #if defined(HAVE_SCHED_SETSCHEDULER)
 	if (!priority_done) {
 		extern int config_priority_override, config_priority;
@@ -466,19 +471,13 @@ ntpdmain(
 
 	{
 		int optct = optionProcess(
-#ifdef SIM
-					  &ntpdsimOptions
-#else
 					  &ntpdOptions
-#endif
 					  , argc, argv);
 		argc -= optct;
 		argv += optct;
 	}
 
-	/* HMS: is this lame? Should we process -l first? */
-
-	init_logging(progname);		/* Open the log file */
+	init_logging(progname, 1);		/* Open the log file */
 
 #ifdef HAVE_UMASK
 	{
@@ -500,21 +499,12 @@ ntpdmain(
 		if (uid)
 		{
 			msyslog(LOG_ERR, "ntpd: must be run as root, not uid %ld", (long)uid);
-			printf("must be run as root, not uid %ld", (long)uid);
+			printf("must be run as root, not uid %ld\n", (long)uid);
 			exit(1);
 		}
 	}
 #endif
 
-#ifdef OPENSSL
-	if ((SSLeay() ^ OPENSSL_VERSION_NUMBER) & ~0xff0L) {
-		msyslog(LOG_ERR,
-		    "ntpd: OpenSSL version mismatch. Built against %lx, you have %lx\n",
-		    OPENSSL_VERSION_NUMBER, SSLeay());
-		exit(1);
-	}
-#endif
-
 	/* getstartup(argc, argv); / * startup configuration, may set debug */
 
 #ifdef DEBUG
@@ -537,18 +527,19 @@ ntpdmain(
 	if (HAVE_OPT( NOVIRTUALIPS ))
 		listen_to_virtual_ips = 0;
 
+	/*
+	 * Limit specific interfaces
+	 */
+	init_specific_interface();
+
 	if (HAVE_OPT( INTERFACE )) {
-#if 0
 		int	ifacect = STACKCT_OPT( INTERFACE );
-		char**	ifaces  = STACKLST_OPT( INTERFACE );
+		const char**	ifaces  = STACKLST_OPT( INTERFACE );
 
 		/* malloc space for the array of names */
 		while (ifacect-- > 0) {
-			next_iface = *ifaces++;
+			add_specific_interface(*ifaces++);
 		}
-#else
-		specific_interface = OPT_ARG( INTERFACE );
-#endif
 	}
 
 	if (HAVE_OPT( NICE ))
@@ -578,14 +569,6 @@ ntpdmain(
 
 	ntp_srandom((int)(now.l_i * now.l_uf));
 
-#ifdef HAVE_DNSREGISTRATION
-	/* HMS: does this have to happen this early? */
-	msyslog(LOG_INFO, "Attemping to register mDNS");
-	if ( DNSServiceRegister (&mdns, 0, 0, NULL, "_ntp._udp", NULL, NULL, htons(NTP_PORT), 0, NULL, NULL, NULL) != kDNSServiceErr_NoError ) {
-		msyslog(LOG_ERR, "Unable to register mDNS");
-	}
-#endif
-
 #if !defined(VMS)
 # ifndef NODETACH
 	/*
@@ -597,6 +580,15 @@ ntpdmain(
 #  endif /* DEBUG */
 	    !nofork)
 	{
+
+	/*
+	 * Install trap handlers to log errors and assertion failures.
+	 * Default handlers print to stderr which doesn't work if detached.
+	 */
+	isc_assertion_setcallback(assertion_failed);
+	isc_error_setfatal(library_fatal_error);
+	isc_error_setunexpected(library_unexpected_error);
+
 #  ifndef SYS_WINNT
 #   ifdef HAVE_DAEMON
 		daemon(0, 0);
@@ -631,6 +623,10 @@ ntpdmain(
 			(void) open("/", 0);
 			(void) dup2(0, 1);
 			(void) dup2(0, 2);
+
+			init_logging(progname, 0);
+			setup_logfile();
+
 #ifdef SYS_DOMAINOS
 			{
 				uid_$t puid;
@@ -732,7 +728,7 @@ ntpdmain(
 	     */
 	    rl.rlim_cur = rl.rlim_max = 32*1024*1024;
 	    if (setrlimit(RLIMIT_MEMLOCK, &rl) == -1) {
-	    	msyslog(LOG_ERR, "Cannot set RLIMIT_MEMLOCK: %m");
+		msyslog(LOG_ERR, "Cannot set RLIMIT_MEMLOCK: %m");
 	    }
 #  endif /* RLIMIT_MEMLOCK */
 	}
@@ -746,7 +742,7 @@ ntpdmain(
 # ifdef HAVE_PLOCK
 #  ifdef PROCLOCK
 #   ifdef _AIX
-	/* 
+	/*
 	 * set the stack limit for AIX for plock().
 	 * see get_aix_stack() for more info.
 	 */
@@ -821,9 +817,6 @@ ntpdmain(
 	init_restrict();
 	init_mon();
 	init_timer();
-#if defined (HAVE_IO_COMPLETION_PORT)
-	init_io_completion_port();
-#endif
 	init_lib();
 	init_request();
 	init_control();
@@ -842,15 +835,18 @@ ntpdmain(
 	 * Get the configuration.  This is done in a separate module
 	 * since this will definitely be different for the gizmo board.
 	 */
-
 	getconfig(argc, argv);
-
-	loop_config(LOOP_DRIFTCOMP, old_drift / 1e6);
-#ifdef OPENSSL
-	crypto_setup();
-#endif /* OPENSSL */
+	report_event(EVNT_SYSRESTART, NULL, NULL);
+	loop_config(LOOP_DRIFTCOMP, old_drift);
 	initializing = 0;
 
+#ifdef HAVE_DNSREGISTRATION
+	msyslog(LOG_INFO, "Attemping to register mDNS");
+	if ( DNSServiceRegister (&mdns, 0, 0, NULL, "_ntp._udp", NULL, NULL, htons(NTP_PORT), 0, NULL, NULL, NULL) != kDNSServiceErr_NoError ) {
+		msyslog(LOG_ERR, "Unable to register mDNS");
+	}
+#endif
+
 #ifdef HAVE_DROPROOT
 	if( droproot ) {
 		/* Drop super-user privileges and chroot now if the OS supports this */
@@ -868,19 +864,37 @@ ntpdmain(
 			exit(-1);
 		}
 #endif /* HAVE_LINUX_CAPABILITIES */
-	
+
 		if (user != NULL) {
 			if (isdigit((unsigned char)*user)) {
 				sw_uid = (uid_t)strtoul(user, &endp, 0);
-				if (*endp != '\0') 
+				if (*endp != '\0')
 					goto getuser;
-			} else {
-getuser:	
-				if ((pw = getpwnam(user)) != NULL) {
-					sw_uid = pw->pw_uid;
+
+				if ((pw = getpwuid(sw_uid)) != NULL) {
+					user = strdup(pw->pw_name);
+					if (NULL == user) {
+						msyslog(LOG_ERR, "strdup() failed: %m");
+						exit (-1);
+					}
+					sw_gid = pw->pw_gid;
 				} else {
 					errno = 0;
-					msyslog(LOG_ERR, "Cannot find user `%s'", user);
+					msyslog(LOG_ERR, "Cannot find user ID %s", user);
+					exit (-1);
+				}
+
+			} else {
+getuser:
+				errno = 0;
+				if ((pw = getpwnam(user)) != NULL) {
+					sw_uid = pw->pw_uid;
+					sw_gid = pw->pw_gid;
+				} else {
+					if (errno)
+					    msyslog(LOG_ERR, "getpwnam(%s) failed: %m", user);
+					else
+					    msyslog(LOG_ERR, "Cannot find user `%s'", user);
 					exit (-1);
 				}
 			}
@@ -888,10 +902,10 @@ getuser:
 		if (group != NULL) {
 			if (isdigit((unsigned char)*group)) {
 				sw_gid = (gid_t)strtoul(group, &endp, 0);
-				if (*endp != '\0') 
+				if (*endp != '\0')
 					goto getgroup;
 			} else {
-getgroup:	
+getgroup:
 				if ((gr = getgrnam(group)) != NULL) {
 					sw_gid = gr->gr_gid;
 				} else {
@@ -901,7 +915,7 @@ getgroup:
 				}
 			}
 		}
-		
+
 		if( chrootdir ) {
 			/* make sure cwd is inside the jail: */
 			if( chdir(chrootdir) ) {
@@ -913,6 +927,10 @@ getgroup:
 				exit (-1);
 			}
 		}
+		if (user && initgroups(user, sw_gid)) {
+			msyslog(LOG_ERR, "Cannot initgroups() to user `%s': %m", user);
+			exit (-1);
+		}
 		if (group && setgid(sw_gid)) {
 			msyslog(LOG_ERR, "Cannot setgid() to group `%s': %m", group);
 			exit (-1);
@@ -929,14 +947,33 @@ getgroup:
 			msyslog(LOG_ERR, "Cannot seteuid() to user `%s': %m", user);
 			exit (-1);
 		}
-	
+
+#ifndef HAVE_LINUX_CAPABILITIES
+		/*
+		 * for now assume that the privilege to bind to privileged ports
+		 * is associated with running with uid 0 - should be refined on
+		 * ports that allow binding to NTP_PORT with uid != 0
+		 */
+		disable_dynamic_updates |= (sw_uid != 0);  /* also notifies routing message listener */
+#endif
+
+		if (disable_dynamic_updates && interface_interval) {
+			interface_interval = 0;
+			msyslog(LOG_INFO, "running in unprivileged mode disables dynamic interface tracking");
+		}
+
 #ifdef HAVE_LINUX_CAPABILITIES
 		do {
-			/*  We may be running under non-root uid now, but we still hold full root privileges!
-			 *  We drop all of them, except for the crucial one: cap_sys_time:
+			/*
+			 *  We may be running under non-root uid now, but we still hold full root privileges!
+			 *  We drop all of them, except for the crucial one or two: cap_sys_time and
+			 *  cap_net_bind_service if doing dynamic interface tracking.
 			 */
 			cap_t caps;
-			if( ! ( caps = cap_from_text( "cap_sys_time=ipe" ) ) ) {
+			char *captext = (interface_interval)
+				? "cap_sys_time,cap_net_bind_service=ipe"
+				: "cap_sys_time=ipe";
+			if( ! ( caps = cap_from_text( captext ) ) ) {
 				msyslog( LOG_ERR, "cap_from_text() failed: %m" );
 				exit(-1);
 			}
@@ -950,11 +987,6 @@ getgroup:
 
 	}    /* if( droproot ) */
 #endif /* HAVE_DROPROOT */
-	
-	/*
-	 * Report that we're up to any trappers
-	 */
-	report_event(EVNT_SYSRESTART, (struct peer *)0);
 
 	/*
 	 * Use select() on all on all input fd's for unlimited
@@ -974,14 +1006,14 @@ getgroup:
 #if defined(HAVE_IO_COMPLETION_PORT)
 
 	for (;;) {
-		int tot_full_recvbufs = GetReceivedBuffers();
+		GetReceivedBuffers();
 #else /* normal I/O */
 
 	BLOCK_IO_AND_ALARM();
 	was_alarmed = 0;
 	for (;;)
 	{
-# if !defined(HAVE_SIGNALED_IO) 
+# if !defined(HAVE_SIGNALED_IO)
 		extern fd_set activefds;
 		extern int maxactivefd;
 
@@ -989,7 +1021,7 @@ getgroup:
 		int nfound;
 # endif
 
-		if (alarm_flag) 	/* alarmed? */
+		if (alarm_flag)		/* alarmed? */
 		{
 			was_alarmed = 1;
 			alarm_flag = 0;
@@ -1030,10 +1062,10 @@ getgroup:
 				netsyslog(LOG_DEBUG, "select(): nfound=%d, error: %m", nfound);
 #  endif /* DEBUG */
 # else /* HAVE_SIGNALED_IO */
-                        
+
 			wait_for_signal();
 # endif /* HAVE_SIGNALED_IO */
-			if (alarm_flag) 	/* alarmed? */
+			if (alarm_flag)		/* alarmed? */
 			{
 				was_alarmed = 1;
 				alarm_flag = 0;
@@ -1049,25 +1081,36 @@ getgroup:
 			 */
 			timer();
 			was_alarmed = 0;
-                        BLOCK_IO_AND_ALARM();
+			BLOCK_IO_AND_ALARM();
 		}
 
-#endif /* HAVE_IO_COMPLETION_PORT */
+#endif /* ! HAVE_IO_COMPLETION_PORT */
 
 #ifdef DEBUG_TIMING
 		{
 			l_fp pts;
 			l_fp tsa, tsb;
 			int bufcount = 0;
-			
+
 			get_systime(&pts);
 			tsa = pts;
 #endif
 			rbuf = get_full_recv_buffer();
 			while (rbuf != NULL)
 			{
+				if (alarm_flag)
+				{
+					was_alarmed = 1;
+					alarm_flag = 0;
+				}
 				UNBLOCK_IO_AND_ALARM();
 
+				if (was_alarmed)
+				{	/* avoid timer starvation during lengthy I/O handling */
+					timer();
+					was_alarmed = 0;
+				}
+
 				/*
 				 * Call the data procedure to handle each received
 				 * packet.
@@ -1120,30 +1163,95 @@ finish(
 	int sig
 	)
 {
-
 	msyslog(LOG_NOTICE, "ntpd exiting on signal %d", sig);
-	write_stats();
 #ifdef HAVE_DNSREGISTRATION
 	if (mdns != NULL)
-	DNSServiceRefDeallocate(mdns);
+		DNSServiceRefDeallocate(mdns);
 #endif
-
-	switch (sig)
-	{
+	switch (sig) {
 # ifdef SIGBUS
-		case SIGBUS:
+	case SIGBUS:
 		printf("\nfinish(SIGBUS)\n");
 		exit(0);
 # endif
-		case 0: 		/* Should never happen... */
+	case 0:			/* Should never happen... */
 		return;
-		default:
+
+	default:
 		exit(0);
 	}
 }
 #endif	/* SIGDIE2 */
 
 
+/* assertion_failed
+ * Redirect trap messages from ISC libraries to syslog.
+ * This code was cloned and simplified from BIND.
+ */
+
+/*
+ * assertion_failed - Handle assertion failures.
+ */
+
+static void
+assertion_failed(const char *file, int line, isc_assertiontype_t type,
+                 const char *cond)
+{
+	isc_assertion_setcallback(NULL);    /* Avoid recursion */
+
+	msyslog(LOG_ERR, "%s:%d: %s(%s) failed",
+		file, line, isc_assertion_typetotext(type), cond);
+	msyslog(LOG_ERR, "exiting (due to assertion failure)");
+
+	abort();
+}
+
+/*
+ * library_fatal_error - Handle fatal errors from our libraries.
+ */
+
+static void
+library_fatal_error(const char *file, int line, const char *format,
+                    va_list args)
+{
+	char errbuf[256];
+
+	isc_error_setfatal(NULL);  /* Avoid recursion */
+
+	msyslog(LOG_ERR, "%s:%d: fatal error:", file, line);
+	vsnprintf(errbuf, sizeof(errbuf), format, args);
+	msyslog(LOG_ERR, errbuf);
+	msyslog(LOG_ERR, "exiting (due to fatal error in library)");
+
+	abort();
+}
+
+/*
+ * library_unexpected_error - Handle non fatal errors from our libraries.
+ */
+#define MAX_UNEXPECTED_ERRORS 100
+int unexpected_error_cnt = 0;
+static void
+library_unexpected_error(const char *file, int line, const char *format,
+                         va_list args)
+{
+	char errbuf[256];
+
+	if (unexpected_error_cnt >= MAX_UNEXPECTED_ERRORS)
+		return;	/* avoid clutter in log */
+
+	msyslog(LOG_ERR, "%s:%d: unexpected error:", file, line);
+	vsnprintf(errbuf, sizeof(errbuf), format, args);
+	msyslog(LOG_ERR, errbuf);
+
+	if (++unexpected_error_cnt == MAX_UNEXPECTED_ERRORS)
+	{
+		msyslog(LOG_ERR, "Too many errors.  Shutting up.");
+	}
+
+}
+
+
 #ifdef DEBUG
 #ifndef SYS_WINNT
 /*
