@@ -65,7 +65,7 @@ struct timeval timeout = {0,0};
  * Windows does not abort a select select call if SIGALRM goes off
  * so a 200 ms timeout is needed
  */
-struct timeval timeout = {0,1000000/TIMER_HZ};
+struct sock_timeval timeout = {0,1000000/TIMER_HZ};
 #else
 struct timeval timeout = {60,0};
 #endif
@@ -84,6 +84,7 @@ struct timeval timeout = {60,0};
 				on Windows NT timers. */
 #pragma comment(lib, "winmm")
 isc_boolean_t ntp_port_inuse(int af, u_short port);
+UINT wTimerRes;
 #endif /* SYS_WINNT */
 
 /*
@@ -104,11 +105,11 @@ static timer_t ntpdate_timerid;
  * Compatibility stuff for Version 2
  */
 #define NTP_MAXSKW	0x28f	/* 0.01 sec in fp format */
-#define NTP_MINDIST 0x51f	/* 0.02 sec in fp format */
+#define NTP_MINDIST	0x51f	/* 0.02 sec in fp format */
 #define PEER_MAXDISP	(64*FP_SECOND)	/* maximum dispersion (fp 64) */
 #define NTP_INFIN	15	/* max stratum, infinity a la Bellman-Ford */
 #define NTP_MAXWGT	(8*FP_SECOND)	/* maximum select weight 8 seconds */
-#define NTP_MAXLIST 5	/* maximum select list size */
+#define NTP_MAXLIST	5	/* maximum select list size */
 #define PEER_SHIFT	8	/* 8 suitable for crystal time base */
 
 /*
@@ -126,8 +127,8 @@ volatile int debug = 0;
  */
 
 int ai_fam_templ;
-int nbsock;
-SOCKET fd[MAX_AF];	/* support up to 2 sockets */
+int nbsock;             /* the number of sockets used */
+SOCKET fd[MAX_AF];
 int fd_family[MAX_AF];	/* to remember the socket family */
 #ifdef HAVE_POLL_H
 struct pollfd fdmask[MAX_AF];
@@ -154,15 +155,10 @@ volatile int alarm_flag = 0;
 int simple_query = 0;
 
 /*
- * Unpriviledged port flag.
+ * Unprivileged port flag.
  */
 int unpriv_port = 0;
 
-/*
- * Time to spend measuring drift rate
- */
-int rate = 0;
-
 /*
  * Program name.
  */
@@ -215,35 +211,34 @@ int verbose = 0;
 int always_step = 0;
 int never_step = 0;
 
-int 	ntpdatemain P((int, char **));
+int 	ntpdatemain (int, char **);
 
-static	void	transmit	P((struct server *));
-static	void	receive 	P((struct recvbuf *));
-static	void	server_data P((struct server *, s_fp, l_fp *, u_fp));
-static	void	clock_filter	P((struct server *));
-static	struct server *clock_select P((void));
-static	int clock_adjust	P((void));
-static	void	addserver	P((char *));
-static	struct server *findserver P((struct sockaddr_storage *));
-		void	timer		P((void));
-static	void	init_alarm	P((void));
+static	void	transmit	(struct server *);
+static	void	receive 	(struct recvbuf *);
+static	void	server_data (struct server *, s_fp, l_fp *, u_fp);
+static	void	clock_filter	(struct server *);
+static	struct server *clock_select (void);
+static	int clock_adjust	(void);
+static	void	addserver	(char *);
+static	struct server *findserver (struct sockaddr_storage *);
+		void	timer		(void);
+static	void	init_alarm	(void);
 #ifndef SYS_WINNT
-static	RETSIGTYPE alarming P((int));
+static	RETSIGTYPE alarming (int);
 #endif /* SYS_WINNT */
-static	void	init_io 	P((void));
-static	void	sendpkt 	P((struct sockaddr_storage *, struct pkt *, int));
-void	input_handler	P((void));
+static	void	init_io 	(void);
+static	void	sendpkt 	(struct sockaddr_storage *, struct pkt *, int);
+void	input_handler	(void);
 
-static	int l_adj_systime	P((l_fp *));
-static	int l_step_systime	P((l_fp *));
+static	int l_adj_systime	(l_fp *);
+static	int l_step_systime	(l_fp *);
 
-static	void	printserver P((struct server *, FILE *));
+static	void	printserver (struct server *, FILE *);
 
 #ifdef SYS_WINNT
 int 	on = 1;
 WORD	wVersionRequested;
-WSADATA wsaData;
-HANDLE	TimerThreadHandle = NULL;
+WSADATA	wsaData;
 #endif /* SYS_WINNT */
 
 #ifdef NO_MAIN_ALLOWED
@@ -274,14 +269,10 @@ void clear_globals()
   simple_query = 0;
 
   /*
-   * Unpriviledged port flag.
+   * Unprivileged port flag.
    */
   unpriv_port = 0;
 
-  /*
-   * Time to spend measuring drift rate
-   */
-  rate = 0;
   /*
    * Systemwide parameters and flags
    */
@@ -307,7 +298,7 @@ void clear_globals()
 #endif
 
 #ifdef HAVE_NETINFO
-static ni_namelist *getnetinfoservers P((void));
+static ni_namelist *getnetinfoservers (void);
 #endif
 
 /*
@@ -376,7 +367,7 @@ ntpdatemain (
 	/*
 	 * Decode argument list
 	 */
-	while ((c = ntp_getopt(argc, argv, "46a:bBde:k:o:p:qr:st:uv")) != EOF)
+	while ((c = ntp_getopt(argc, argv, "46a:bBde:k:o:p:qst:uv")) != EOF)
 		switch (c)
 		{
 		case '4':
@@ -432,17 +423,6 @@ ntpdatemain (
 		case 'q':
 			simple_query = 1;
 			break;
-		case 'r':
-			c = atoi(ntp_optarg);
-			if (c <= 0 || c > (60 * 60)) {
-				(void) fprintf(stderr,
-					   "%s: rate (%d) is invalid: 0 - %d\n",
-					   progname, c, (60 * 60));
-				errflg++;
-			} else {
-				rate = c;
-			}
-			break;
 		case 's':
 			syslogit = 1;
 			break;
@@ -474,7 +454,7 @@ ntpdatemain (
 	
 	if (errflg) {
 		(void) fprintf(stderr,
-		    "usage: %s [-46bBdqsuv] [-a key#] [-e delay] [-k file] [-p samples] [-o version#] [-r rate] [-t timeo] server ...\n",
+		    "usage: %s [-46bBdqsuv] [-a key#] [-e delay] [-k file] [-p samples] [-o version#] [-t timeo] server ...\n",
 		    progname);
 		exit(2);
 	}
@@ -482,12 +462,7 @@ ntpdatemain (
 	if (debug || simple_query) {
 #ifdef HAVE_SETVBUF
 		static char buf[BUFSIZ];
-#ifdef SYS_WINNT
-		/* Win32 does not implement line bufferibg */
-		setvbuf(stdout, NULL, _IONBF, BUFSIZ);
-#else
 		setvbuf(stdout, buf, _IOLBF, BUFSIZ);
-#endif	/* SYS_WINNT */
 #else
 		setlinebuf(stdout);
 #endif
@@ -560,10 +535,8 @@ ntpdatemain (
 		}
 		authtrust(sys_authkey, 1);
 		if (!authistrusted(sys_authkey)) {
-			char buf[10];
-
-			(void) sprintf(buf, "%lu", (unsigned long)sys_authkey);
-			msyslog(LOG_ERR, "authentication key %s unknown", buf);
+			msyslog(LOG_ERR, "authentication key %lu unknown",
+				(unsigned long) sys_authkey);
 			exit(1);
 		}
 	}
@@ -582,13 +555,6 @@ ntpdatemain (
 #if defined(HAVE_BSD_NICE)
 	(void) setpriority(PRIO_PROCESS, 0, NTPDATE_PRIO);
 #endif
-#ifdef SYS_WINNT
-	process_handle = GetCurrentProcess();
-	if (!SetPriorityClass(process_handle, (DWORD) REALTIME_PRIORITY_CLASS)) {
-		msyslog(LOG_ERR, "SetPriorityClass failed: %m");
-	}
-#endif /* SYS_WINNT */
-
 
 
 	initializing = 0;
@@ -740,7 +706,7 @@ transmit(
 	xpkt.ppoll = NTP_MINPOLL;
 	xpkt.precision = NTPDATE_PRECISION;
 	xpkt.rootdelay = htonl(NTPDATE_DISTANCE);
-	xpkt.rootdispersion = htonl(NTPDATE_DISP);
+	xpkt.rootdisp = htonl(NTPDATE_DISP);
 	xpkt.refid = htonl(NTPDATE_REFID);
 	L_CLR(&xpkt.reftime);
 	L_CLR(&xpkt.org);
@@ -885,7 +851,7 @@ receive(
 	server->stratum = PKT_TO_STRATUM(rpkt->stratum);
 	server->precision = rpkt->precision;
 	server->rootdelay = ntohl(rpkt->rootdelay);
-	server->rootdispersion = ntohl(rpkt->rootdispersion);
+	server->rootdisp = ntohl(rpkt->rootdisp);
 	server->refid = rpkt->refid;
 	NTOHL_FP(&rpkt->reftime, &server->reftime);
 	NTOHL_FP(&rpkt->rec, &rec);
@@ -1304,7 +1270,7 @@ clock_adjust(void)
 	}
 
 	if (dostep) {
-		if (simple_query || l_step_systime(&server->offset)) {
+		if (simple_query || debug || l_step_systime(&server->offset)){
 			msyslog(LOG_NOTICE, "step time server %s offset %s sec",
 				stoa(&server->srcadr),
 				lfptoa(&server->offset, 6));
@@ -1391,6 +1357,15 @@ addserver(
 
 	error = getaddrinfo(serv, service, &hints, &addrResult);
 	if (error != 0) {
+		/* Conduct more refined error analysis */
+		if (error == EAI_FAIL || error == EAI_AGAIN){
+			/* Name server is unusable. Exit after failing on the
+			   first server, in order to shorten the timeout caused
+			   by waiting for resolution of several servers */
+			fprintf(stderr, "Name server cannot be used, exiting");
+			msyslog(LOG_ERR, "name server cannot be used, reason: %s\n", gai_strerror(error));
+			exit(1);
+		}
 		fprintf(stderr, "Error : %s\n", gai_strerror(error));
 		msyslog(LOG_ERR, "can't find host %s\n", serv);
 		return;
@@ -1542,8 +1517,18 @@ alarming(
 void CALLBACK 
 alarming(UINT uTimerID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
 {
+	UNUSED_ARG(uTimerID); UNUSED_ARG(uMsg); UNUSED_ARG(dwUser);
+	UNUSED_ARG(dw1); UNUSED_ARG(dw2);
+
 	alarm_flag++;
 }
+
+static void
+callTimeEndPeriod(void)
+{
+	timeEndPeriod( wTimerRes );
+	wTimerRes = 0;
+}
 #endif /* SYS_WINNT */
 
 
@@ -1561,7 +1546,7 @@ init_alarm(void)
 # endif
 #else
 	TIMECAPS tc;
-	UINT wTimerRes, wTimerID;
+	UINT wTimerID;
 # endif /* SYS_WINNT */
 #if defined SYS_CYGWIN32 || defined SYS_WINNT
 	HANDLE hToken;
@@ -1613,7 +1598,7 @@ init_alarm(void)
 # endif
 #if defined SYS_CYGWIN32
 	/*
-	 * Get previleges needed for fiddling with the clock
+	 * Get privileges needed for fiddling with the clock
 	 */
 
 	/* get the current process token handle */
@@ -1635,7 +1620,7 @@ init_alarm(void)
 	_tzset();
 
 	/*
-	 * Get previleges needed for fiddling with the clock
+	 * Get privileges needed for fiddling with the clock
 	 */
 
 	/* get the current process token handle */
@@ -1668,6 +1653,7 @@ init_alarm(void)
 	wTimerRes = min(max(tc.wPeriodMin, TARGET_RESOLUTION), tc.wPeriodMax);
 	/* establish the minimum timer resolution that we'll use */
 	timeBeginPeriod(wTimerRes);
+	atexit(callTimeEndPeriod);
 
 	/* start the timer event */
 	wTimerID = timeSetEvent(
@@ -1710,6 +1696,7 @@ init_io(void)
 	struct addrinfo hints;
 	char service[5];
 	int optval = 1;
+	int check_ntp_port_in_use = !debug && !simple_query && !unpriv_port;
 
 	/*
 	 * Init buffer free list and stat counters
@@ -1737,7 +1724,7 @@ init_io(void)
 	}
 
 #ifdef SYS_WINNT
-	if (ntp_port_inuse(AF_INET, NTP_PORT) ){
+	if (check_ntp_port_in_use && ntp_port_inuse(AF_INET, NTP_PORT)){
 		netsyslog(LOG_ERR, "the NTP socket is in use, exiting: %m");
 		exit(1);
 	}
@@ -1751,8 +1738,8 @@ init_io(void)
 	 */
 	for(nbsock = 0; (nbsock < MAX_AF) && res ; res = res->ai_next) {
 	/* create a datagram (UDP) socket */
-	   fd[nbsock] = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
-	   if (fd[nbsock] == SOCKET_ERROR) {
+		fd[nbsock] = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+		if (fd[nbsock] == SOCKET_ERROR) {
 #ifndef SYS_WINNT
 		if (errno == EPROTONOSUPPORT || errno == EAFNOSUPPORT ||
 		    errno == EPFNOSUPPORT)
@@ -1765,95 +1752,94 @@ init_io(void)
 		netsyslog(LOG_ERR, "socket() failed: %m");
 		exit(1);
 		/*NOTREACHED*/
-	   }
-	   /* set socket to reuse address */
-	   if (setsockopt(fd[nbsock], SOL_SOCKET, SO_REUSEADDR, (void*) &optval, sizeof(optval)) < 0) {
-		   netsyslog(LOG_ERR, "setsockopt() SO_REUSEADDR failed: %m");
-		   exit(1);
-		   /*NOTREACHED*/
-	   }
+		}
+		/* set socket to reuse address */
+		if (setsockopt(fd[nbsock], SOL_SOCKET, SO_REUSEADDR, (void*) &optval, sizeof(optval)) < 0) {
+				netsyslog(LOG_ERR, "setsockopt() SO_REUSEADDR failed: %m");
+				exit(1);
+				/*NOTREACHED*/
+		}
 #ifdef IPV6_V6ONLY
-	   /* Restricts AF_INET6 socket to IPv6 communications (see RFC 2553bis-03) */
-	   if (res->ai_family == AF_INET6)
-		if (setsockopt(fd[nbsock], IPPROTO_IPV6, IPV6_V6ONLY, (void*) &optval, sizeof(optval)) < 0) {
-			   netsyslog(LOG_ERR, "setsockopt() IPV6_V6ONLY failed: %m");
-			   exit(1);
-			   /*NOTREACHED*/
+		/* Restricts AF_INET6 socket to IPv6 communications (see RFC 2553bis-03) */
+		if (res->ai_family == AF_INET6)
+			if (setsockopt(fd[nbsock], IPPROTO_IPV6, IPV6_V6ONLY, (void*) &optval, sizeof(optval)) < 0) {
+				   netsyslog(LOG_ERR, "setsockopt() IPV6_V6ONLY failed: %m");
+					exit(1);
+					/*NOTREACHED*/
 		}
 #endif
 
-	   /* Remember the socket family in fd_family structure */
-	   fd_family[nbsock] = res->ai_family;
+		/* Remember the socket family in fd_family structure */
+		fd_family[nbsock] = res->ai_family;
 
-	/*
-	 * bind the socket to the NTP port
-	 */
-	if (!debug && !simple_query && !unpriv_port) {
-		if (bind(fd[nbsock], res->ai_addr, SOCKLEN(res->ai_addr)) < 0) {
+		/*
+		 * bind the socket to the NTP port
+		 */
+		if (check_ntp_port_in_use) {
+			if (bind(fd[nbsock], res->ai_addr, SOCKLEN(res->ai_addr)) < 0) {
 #ifndef SYS_WINNT
-			if (errno == EADDRINUSE)
+				if (errno == EADDRINUSE)
 #else
 				if (WSAGetLastError() == WSAEADDRINUSE)
 #endif /* SYS_WINNT */
-				netsyslog(LOG_ERR,
-					"the NTP socket is in use, exiting");
+					netsyslog(LOG_ERR, "the NTP socket is in use, exiting");
 				else
-				netsyslog(LOG_ERR, "bind() fails: %m");
-			exit(1);
+					netsyslog(LOG_ERR, "bind() fails: %m");
+				exit(1);
+			}
 		}
-	}
 
 #ifdef HAVE_POLL_H
-	    fdmask[nbsock].fd = fd[nbsock];
-	    fdmask[nbsock].events = POLLIN;
+		fdmask[nbsock].fd = fd[nbsock];
+		fdmask[nbsock].events = POLLIN;
 #else
-	    FD_SET(fd[nbsock], &fdmask);
-	    if (maxfd < fd[nbsock]+1) {
-		maxfd = fd[nbsock]+1;
-	    }
+		FD_SET(fd[nbsock], &fdmask);
+		if (maxfd < fd[nbsock]+1) {
+			maxfd = fd[nbsock]+1;
+		}
 #endif
 
-	/*
-	 * set non-blocking,
-	 */
+		/*
+		 * set non-blocking,
+		 */
 #ifndef SYS_WINNT
 # ifdef SYS_VXWORKS
-  {
-	int on = TRUE;
+		{
+		int on = TRUE;
 
-	if (ioctl(fd[nbsock],FIONBIO, &on) == ERROR) {
-	  netsyslog(LOG_ERR, "ioctl(FIONBIO) fails: %m");
-	  exit(1);
-	}
-  }
+		if (ioctl(fd[nbsock],FIONBIO, &on) == ERROR) {
+		  netsyslog(LOG_ERR, "ioctl(FIONBIO) fails: %m");
+			exit(1);
+		}
+		}
 # else /* not SYS_VXWORKS */
 #  if defined(O_NONBLOCK)
-	   if (fcntl(fd[nbsock], F_SETFL, O_NONBLOCK) < 0) {
-		netsyslog(LOG_ERR, "fcntl(FNDELAY|FASYNC) fails: %m");
-		exit(1);
-		/*NOTREACHED*/
-	}
+		if (fcntl(fd[nbsock], F_SETFL, O_NONBLOCK) < 0) {
+			netsyslog(LOG_ERR, "fcntl(FNDELAY|FASYNC) fails: %m");
+			exit(1);
+			/*NOTREACHED*/
+		}
 #  else /* not O_NONBLOCK */
 #	if defined(FNDELAY)
-	   if (fcntl(fd[nbsock], F_SETFL, FNDELAY) < 0) {
-		netsyslog(LOG_ERR, "fcntl(FNDELAY|FASYNC) fails: %m");
-		exit(1);
-		/*NOTREACHED*/
-	}
+		if (fcntl(fd[nbsock], F_SETFL, FNDELAY) < 0) {
+			netsyslog(LOG_ERR, "fcntl(FNDELAY|FASYNC) fails: %m");
+			exit(1);
+			/*NOTREACHED*/
+		}
 #	else /* FNDELAY */
 #	 include "Bletch: Need non blocking I/O"
 #	endif /* FNDELAY */
 #  endif /* not O_NONBLOCK */
 # endif /* SYS_VXWORKS */
 #else /* SYS_WINNT */
-	if (ioctlsocket(fd[nbsock], FIONBIO, (u_long *) &on) == SOCKET_ERROR) {
-		netsyslog(LOG_ERR, "ioctlsocket(FIONBIO) fails: %m");
-		exit(1);
-	}
+		if (ioctlsocket(fd[nbsock], FIONBIO, (u_long *) &on) == SOCKET_ERROR) {
+			netsyslog(LOG_ERR, "ioctlsocket(FIONBIO) fails: %m");
+			exit(1);
+		}
 #endif /* SYS_WINNT */
-	nbsock++;
-    }
-    freeaddrinfo(ressave);
+		nbsock++;
+	}
+	freeaddrinfo(ressave);
 }
 
 /*
@@ -1911,10 +1897,10 @@ input_handler(void)
 {
 	register int n;
 	register struct recvbuf *rb;
-	struct timeval tvzero;
+	struct sock_timeval tvzero;
 	int fromlen;
 	l_fp ts;
-	SOCKET i;
+	int i;
 #ifdef HAVE_POLL_H
 	struct pollfd fds[MAX_AF];
 #else
@@ -1950,7 +1936,7 @@ input_handler(void)
 		 * Determine which socket received data
 		 */
 
-		for(i=0; i < maxfd; i++) {
+		for(i=0; i < nbsock; i++) {
 			if(FD_ISSET(fd[i], &fds)) {
 				 fdc = fd[i];
 				 break;
