@@ -20,21 +20,28 @@
 #include "ntpdc-opts.h"
 
 #ifdef SYS_WINNT
-#include <Mswsock.h>
+# include <Mswsock.h>
 # include <io.h>
-#else
-# define closesocket close
 #endif /* SYS_WINNT */
 
-#if defined(HAVE_LIBREADLINE) || defined (HAVE_LIBEDIT)
+#if defined(HAVE_LIBREADLINE)
 # include <readline/readline.h>
 # include <readline/history.h>
-#endif /* HAVE_LIBREADLINE || HAVE_LIBEDIT */
+#endif /* HAVE_LIBREADLINE */
+
+#if defined (HAVE_LIBEDIT)
+# include <histedit.h>
+#endif /* HAVE_LIBEDIT */
 
 #ifdef SYS_VXWORKS
-/* vxWorks needs mode flag -casey*/
-#define open(name, flags)   open(name, flags, 0777)
-#define SERVER_PORT_NUM     123
+				/* vxWorks needs mode flag -casey*/
+# define open(name, flags)   open(name, flags, 0777)
+# define SERVER_PORT_NUM     123
+#endif
+
+/* We use COMMAND as an autogen keyword */
+#ifdef COMMAND
+# undef COMMAND
 #endif
 
 /*
@@ -64,42 +71,42 @@ u_long	current_time;		/* needed by authkeys; not used */
  */
 s_char	sys_precision;		/* local clock precision (log2 s) */
 
-int		ntpdcmain	P((int,	char **));
+int		ntpdcmain	(int,	char **);
 /*
  * Built in command handler declarations
  */
-static	int	openhost	P((const char *));
-static	int	sendpkt		P((char *, int));
-static	void	growpktdata	P((void));
-static	int	getresponse	P((int, int, int *, int *, char **, int));
-static	int	sendrequest	P((int, int, int, int, int, char *));
-static	void	getcmds		P((void));
-static	RETSIGTYPE abortcmd	P((int));
-static	void	docmd		P((const char *));
-static	void	tokenize	P((const char *, char **, int *));
-static	int	findcmd		P((char *, struct xcmd *, struct xcmd *, struct xcmd **));
-static	int	getarg		P((char *, int, arg_v *));
-static	int	getnetnum	P((const char *, struct sockaddr_storage *, char *, int));
-static	void	help		P((struct parse *, FILE *));
+static	int	openhost	(const char *);
+static	int	sendpkt		(char *, int);
+static	void	growpktdata	(void);
+static	int	getresponse	(int, int, int *, int *, char **, int);
+static	int	sendrequest	(int, int, int, int, int, char *);
+static	void	getcmds		(void);
+static	RETSIGTYPE abortcmd	(int);
+static	void	docmd		(const char *);
+static	void	tokenize	(const char *, char **, int *);
+static	int	findcmd		(char *, struct xcmd *, struct xcmd *, struct xcmd **);
+static	int	getarg		(char *, int, arg_v *);
+static	int	getnetnum	(const char *, struct sockaddr_storage *, char *, int);
+static	void	help		(struct parse *, FILE *);
 #ifdef QSORT_USES_VOID_P
-static	int	helpsort	P((const void *, const void *));
+static	int	helpsort	(const void *, const void *);
 #else
-static	int	helpsort	P((char **, char **));
+static	int	helpsort	(char **, char **);
 #endif
-static	void	printusage	P((struct xcmd *, FILE *));
-static	void	timeout		P((struct parse *, FILE *));
-static	void	my_delay	P((struct parse *, FILE *));
-static	void	host		P((struct parse *, FILE *));
-static	void	keyid		P((struct parse *, FILE *));
-static	void	keytype		P((struct parse *, FILE *));
-static	void	passwd		P((struct parse *, FILE *));
-static	void	hostnames	P((struct parse *, FILE *));
-static	void	setdebug	P((struct parse *, FILE *));
-static	void	quit		P((struct parse *, FILE *));
-static	void	version		P((struct parse *, FILE *));
-static	void	warning		P((const char *, const char *, const char *));
-static	void	error		P((const char *, const char *, const char *));
-static	u_long	getkeyid	P((const char *));
+static	void	printusage	(struct xcmd *, FILE *);
+static	void	timeout		(struct parse *, FILE *);
+static	void	my_delay	(struct parse *, FILE *);
+static	void	host		(struct parse *, FILE *);
+static	void	keyid		(struct parse *, FILE *);
+static	void	keytype		(struct parse *, FILE *);
+static	void	passwd		(struct parse *, FILE *);
+static	void	hostnames	(struct parse *, FILE *);
+static	void	setdebug	(struct parse *, FILE *);
+static	void	quit		(struct parse *, FILE *);
+static	void	version		(struct parse *, FILE *);
+static	void	warning		(const char *, const char *, const char *);
+static	void	error		(const char *, const char *, const char *);
+static	u_long	getkeyid	(const char *);
 
 
 
@@ -168,8 +175,8 @@ static	struct xcmd builtins[] = {
 /*
  * Some variables used and manipulated locally
  */
-static	struct timeval tvout = { DEFTIMEOUT, 0 };	/* time out for reads */
-static	struct timeval tvsout = { DEFSTIMEOUT, 0 };	/* secondary time out */
+static	struct sock_timeval tvout = { DEFTIMEOUT, 0 };	/* time out for reads */
+static	struct sock_timeval tvsout = { DEFSTIMEOUT, 0 };/* secondary time out */
 static	l_fp delay_time;				/* delay time */
 static	char currenthost[LENHOSTNAME];			/* current host name */
 int showhostnames = 1;					/* show host names by default */
@@ -180,18 +187,6 @@ static	SOCKET sockfd;					/* fd socket is opened on */
 static	int havehost = 0;				/* set to 1 when host open */
 int s_port = 0;
 
-#if defined (SYS_WINNT) || defined (SYS_VXWORKS)
-char password[9];
-#endif /* SYS_WINNT || SYS_VXWORKS */
-
-#ifdef SYS_WINNT
-DWORD NumberOfBytesWritten;
-
-HANDLE	TimerThreadHandle = NULL;	/* 1998/06/03 - Used in ntplib/machines.c */
-void timer(void)	{  ; };	/* 1998/06/03 - Used in ntplib/machines.c */
-
-#endif /* SYS_WINNT */
-
 /*
  * Holds data returned from queries.  We allocate INITDATASIZE
  * octets to begin with, increasing this as we need to.
@@ -291,6 +286,7 @@ ntpdcmain(
 	char *argv[]
 	)
 {
+	extern int ntp_optind;
 
 	delay_time.l_ui = 0;
 	delay_time.l_uf = DEFDELAY;
@@ -364,6 +360,18 @@ ntpdcmain(
 		ADDCMD("dmpeers");
 	}
 
+	if (ntp_optind == argc) {
+		ADDHOST(DEFHOST);
+	} else {
+		for (; ntp_optind < argc; ntp_optind++)
+		    ADDHOST(argv[ntp_optind]);
+	}
+
+	if (numcmds == 0 && interactive == 0
+	    && isatty(fileno(stdin)) && isatty(fileno(stderr))) {
+		interactive = 1;
+	}
+
 #if 0
 	ai_fam_templ = ai_fam_default;
 	while ((c = ntp_getopt(argc, argv, "46c:dilnps")) != EOF)
@@ -399,12 +407,14 @@ ntpdcmain(
 		    errflg++;
 		    break;
 	    }
+
 	if (errflg) {
 		(void) fprintf(stderr,
 			       "usage: %s [-46dilnps] [-c cmd] host ...\n",
 			       progname);
 		exit(2);
 	}
+
 	if (ntp_optind == argc) {
 		ADDHOST(DEFHOST);
 	} else {
@@ -479,10 +489,14 @@ openhost(
 	
 	if (*cp == '[') {
 		cp++;	
-		for(i = 0; *cp != ']'; cp++, i++)
-			name[i] = *cp;	
-		name[i] = '\0';
-		hname = name;
+		for (i = 0; *cp && *cp != ']'; cp++, i++)
+			name[i] = *cp;
+		if (*cp == ']') {
+			name[i] = '\0';
+			hname = name;
+		} else {
+			return 0;
+		}
 	}	
 
 	/*
@@ -656,7 +670,7 @@ getresponse(
 	)
 {
 	struct resp_pkt rpkt;
-	struct timeval tvo;
+	struct sock_timeval tvo;
 	int items;
 	int i;
 	int size;
@@ -987,7 +1001,7 @@ doquery(
 	int res;
 	char junk[512];
 	fd_set fds;
-	struct timeval tvzero;
+	struct sock_timeval tvzero;
 
 	/*
 	 * Check to make sure host is open
@@ -1104,7 +1118,7 @@ again:
 static void
 getcmds(void)
 {
-#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDIT)
+#if defined(HAVE_LIBREADLINE)
 	char *line;
 
 	for (;;) {
@@ -1113,7 +1127,7 @@ getcmds(void)
 		docmd(line);
 		free(line);
 	}
-#else /* not (HAVE_LIBREADLINE || HAVE_LIBEDIT) */
+#else /* not (HAVE_LIBREADLINE) */
 	char line[MAXLINE];
 
 	for (;;) {
