@@ -27,59 +27,59 @@
 /*
  * Declarations for command handlers in here
  */
-static	int	checkitems	P((int, FILE *));
-static	int	checkitemsize	P((int, int));
-static	int	check1item	P((int, FILE *));
-static	void	peerlist	P((struct parse *, FILE *));
-static	void	peers		P((struct parse *, FILE *));
-static void	doconfig	P((struct parse *pcmd, FILE *fp, int mode, int refc));
-static	void	dmpeers		P((struct parse *, FILE *));
-static	void	dopeers		P((struct parse *, FILE *, int));
-static	void	printpeer	P((struct info_peer *, FILE *));
-static	void	showpeer	P((struct parse *, FILE *));
-static	void	peerstats	P((struct parse *, FILE *));
-static	void	loopinfo	P((struct parse *, FILE *));
-static	void	sysinfo		P((struct parse *, FILE *));
-static	void	sysstats	P((struct parse *, FILE *));
-static	void	iostats		P((struct parse *, FILE *));
-static	void	memstats	P((struct parse *, FILE *));
-static	void	timerstats	P((struct parse *, FILE *));
-static	void	addpeer		P((struct parse *, FILE *));
-static	void	addserver	P((struct parse *, FILE *));
-static	void	addrefclock	P((struct parse *, FILE *));
-static	void	broadcast	P((struct parse *, FILE *));
-static	void	doconfig	P((struct parse *, FILE *, int, int));
-static	void	unconfig	P((struct parse *, FILE *));
-static	void	set		P((struct parse *, FILE *));
-static	void	sys_clear	P((struct parse *, FILE *));
-static	void	doset		P((struct parse *, FILE *, int));
-static	void	reslist		P((struct parse *, FILE *));
-static	void	new_restrict	P((struct parse *, FILE *));
-static	void	unrestrict	P((struct parse *, FILE *));
-static	void	delrestrict	P((struct parse *, FILE *));
-static	void	do_restrict	P((struct parse *, FILE *, int));
-static	void	monlist		P((struct parse *, FILE *));
-static	void	reset		P((struct parse *, FILE *));
-static	void	preset		P((struct parse *, FILE *));
-static	void	readkeys	P((struct parse *, FILE *));
-static	void	trustkey	P((struct parse *, FILE *));
-static	void	untrustkey	P((struct parse *, FILE *));
-static	void	do_trustkey	P((struct parse *, FILE *, int));
-static	void	authinfo	P((struct parse *, FILE *));
-static	void	traps		P((struct parse *, FILE *));
-static	void	addtrap		P((struct parse *, FILE *));
-static	void	clrtrap		P((struct parse *, FILE *));
-static	void	do_addclr_trap	P((struct parse *, FILE *, int));
-static	void	requestkey	P((struct parse *, FILE *));
-static	void	controlkey	P((struct parse *, FILE *));
-static	void	do_changekey	P((struct parse *, FILE *, int));
-static	void	ctlstats	P((struct parse *, FILE *));
-static	void	clockstat	P((struct parse *, FILE *));
-static	void	fudge		P((struct parse *, FILE *));
-static	void	clkbug		P((struct parse *, FILE *));
-static	void	kerninfo	P((struct parse *, FILE *));
-static  void    get_if_stats    P((struct parse *, FILE *));
-static  void    do_if_reload    P((struct parse *, FILE *));
+static	int	checkitems	(int, FILE *);
+static	int	checkitemsize	(int, int);
+static	int	check1item	(int, FILE *);
+static	void	peerlist	(struct parse *, FILE *);
+static	void	peers		(struct parse *, FILE *);
+static void	doconfig	(struct parse *pcmd, FILE *fp, int mode, int refc);
+static	void	dmpeers		(struct parse *, FILE *);
+static	void	dopeers		(struct parse *, FILE *, int);
+static	void	printpeer	(struct info_peer *, FILE *);
+static	void	showpeer	(struct parse *, FILE *);
+static	void	peerstats	(struct parse *, FILE *);
+static	void	loopinfo	(struct parse *, FILE *);
+static	void	sysinfo		(struct parse *, FILE *);
+static	void	sysstats	(struct parse *, FILE *);
+static	void	iostats		(struct parse *, FILE *);
+static	void	memstats	(struct parse *, FILE *);
+static	void	timerstats	(struct parse *, FILE *);
+static	void	addpeer		(struct parse *, FILE *);
+static	void	addserver	(struct parse *, FILE *);
+static	void	addrefclock	(struct parse *, FILE *);
+static	void	broadcast	(struct parse *, FILE *);
+static	void	doconfig	(struct parse *, FILE *, int, int);
+static	void	unconfig	(struct parse *, FILE *);
+static	void	set		(struct parse *, FILE *);
+static	void	sys_clear	(struct parse *, FILE *);
+static	void	doset		(struct parse *, FILE *, int);
+static	void	reslist		(struct parse *, FILE *);
+static	void	new_restrict	(struct parse *, FILE *);
+static	void	unrestrict	(struct parse *, FILE *);
+static	void	delrestrict	(struct parse *, FILE *);
+static	void	do_restrict	(struct parse *, FILE *, int);
+static	void	monlist		(struct parse *, FILE *);
+static	void	reset		(struct parse *, FILE *);
+static	void	preset		(struct parse *, FILE *);
+static	void	readkeys	(struct parse *, FILE *);
+static	void	trustkey	(struct parse *, FILE *);
+static	void	untrustkey	(struct parse *, FILE *);
+static	void	do_trustkey	(struct parse *, FILE *, int);
+static	void	authinfo	(struct parse *, FILE *);
+static	void	traps		(struct parse *, FILE *);
+static	void	addtrap		(struct parse *, FILE *);
+static	void	clrtrap		(struct parse *, FILE *);
+static	void	do_addclr_trap	(struct parse *, FILE *, int);
+static	void	requestkey	(struct parse *, FILE *);
+static	void	controlkey	(struct parse *, FILE *);
+static	void	do_changekey	(struct parse *, FILE *, int);
+static	void	ctlstats	(struct parse *, FILE *);
+static	void	clockstat	(struct parse *, FILE *);
+static	void	fudge		(struct parse *, FILE *);
+static	void	clkbug		(struct parse *, FILE *);
+static	void	kerninfo	(struct parse *, FILE *);
+static  void    get_if_stats    (struct parse *, FILE *);
+static  void    do_if_reload    (struct parse *, FILE *);
 
 /*
  * Commands we understand.  Ntpdc imports this.
@@ -119,10 +119,10 @@ struct xcmd opcmds[] = {
 	  { "", "", "", "" },
 	  "display event timer subsystem statistics" },
 	{ "addpeer",	addpeer,	{ NTP_ADD, OPT|NTP_STR, OPT|NTP_STR, OPT|NTP_STR },
-	  { "addr", "keyid", "version", "minpoll#|prefer|burst|iburst|dynamic|'minpoll N'|'maxpoll N'|'keyid N'|'version N' ..." },
+	  { "addr", "keyid", "version", "minpoll#|prefer|burst|iburst|'minpoll N'|'maxpoll N'|'keyid N'|'version N' ..." },
 	  "configure a new peer association" },
 	{ "addserver",	addserver,	{ NTP_ADD, OPT|NTP_STR, OPT|NTP_STR, OPT|NTP_STR },
-	  { "addr", "keyid", "version", "minpoll#|prefer|burst|iburst|dynamic|'minpoll N'|'maxpoll N'|'keyid N'|'version N' ..." },
+	  { "addr", "keyid", "version", "minpoll#|prefer|burst|iburst|'minpoll N'|'maxpoll N'|'keyid N'|'version N' ..." },
 	  "configure a new server" },
 	{ "addrefclock",addrefclock,	{ NTP_ADD, OPT|NTP_UINT, OPT|NTP_STR, OPT|NTP_STR },
 	  { "addr", "mode", "minpoll|prefer", "minpoll|prefer" },
@@ -886,7 +886,7 @@ again:
 			       "offset %s, frequency %s, time_const %ld, watchdog %ld\n",
 			       lfptoa(&tempts, 6),
 			       lfptoa(&temp2ts, 3),
-			       (u_long)ntohl((u_long)il->compliance),
+			       (long)(int32_t)ntohl((u_long)il->compliance),
 			       (u_long)ntohl((u_long)il->watchdog_timer));
 	} else {
 		NTOHL_FP(&il->last_offset, &tempts);
@@ -896,7 +896,7 @@ again:
 		(void) fprintf(fp, "frequency:            %s ppm\n",
 			       lfptoa(&tempts, 3));
 		(void) fprintf(fp, "poll adjust:          %ld\n",
-			       (u_long)ntohl(il->compliance));
+			       (long)(int32_t)ntohl(il->compliance));
 		(void) fprintf(fp, "watchdog timer:       %ld s\n",
 			       (u_long)ntohl(il->watchdog_timer));
 	}
@@ -1051,7 +1051,7 @@ again:
 	       (u_long)ntohl(ss->newversionpkt));
 	fprintf(fp, "previous version:       %ld\n",
 	       (u_long)ntohl(ss->oldversionpkt));
-	fprintf(fp, "bad version:            %ld\n",
+	fprintf(fp, "declined:               %ld\n",
 	       (u_long)ntohl(ss->unknownversion));
 	fprintf(fp, "access denied:          %ld\n",
 		(u_long)ntohl(ss->denied));
@@ -1332,8 +1332,6 @@ again:
 		    flags |= CONF_FLAG_PREFER;
 		else if (STREQ(pcmd->argval[items].string, "burst"))
 		    flags |= CONF_FLAG_BURST;
-		else if (STREQ(pcmd->argval[items].string, "dynamic"))
-                    flags |= CONF_FLAG_DYNAMIC;
 		else if (STREQ(pcmd->argval[items].string, "iburst"))
 		    flags |= CONF_FLAG_IBURST;
 		else if (!refc && STREQ(pcmd->argval[items].string, "keyid"))
@@ -2700,7 +2698,6 @@ clockstat(
 	int res;
 	l_fp ts;
 	struct clktype *clk;
-	u_long ltemp;
 
 	for (qitems = 0; qitems < min(pcmd->nargs, 8); qitems++)
 	    clist[qitems] = GET_INADDR(pcmd->argval[qitems].netnum);
@@ -2758,9 +2755,8 @@ again:
 			       lfptoa(&ts, 6));
 		(void) fprintf(fp, "stratum:              %ld\n",
 			       (u_long)ntohl(cl->fudgeval1));
-		ltemp = ntohl(cl->fudgeval2);
 		(void) fprintf(fp, "reference ID:         %s\n",
-			       (char *)&ltemp);
+			       refid_string(ntohl(cl->fudgeval2), 0));
 		(void) fprintf(fp, "fudge flags:          0x%x\n",
 			       cl->flags);
 
@@ -2992,7 +2988,7 @@ again:
 		tscale = 1e-9;
 #endif
 	(void)fprintf(fp, "pll offset:           %g s\n",
-	    (long)ntohl(ik->offset) * tscale);
+	    (int32_t)ntohl(ik->offset) * tscale);
 	(void)fprintf(fp, "pll frequency:        %s ppm\n",
 	    fptoa((s_fp)ntohl(ik->freq), 3));
 	(void)fprintf(fp, "maximum error:        %g s\n",
