@@ -18,51 +18,53 @@ int 	maxhostlen;
 /*
  * Declarations for command handlers in here
  */
-static	int checkassocid	P((u_int32));
-static	char *	strsave 	P((char *));
-static	struct varlist *findlistvar P((struct varlist *, char *));
-static	void	doaddvlist	P((struct varlist *, char *));
-static	void	dormvlist	P((struct varlist *, char *));
-static	void	doclearvlist	P((struct varlist *));
-static	void	makequerydata	P((struct varlist *, int *, char *));
-static	int doquerylist P((struct varlist *, int, int, int, u_short *, int *, char **));
-static	void	doprintvlist	P((struct varlist *, FILE *));
-static	void	addvars 	P((struct parse *, FILE *));
-static	void	rmvars		P((struct parse *, FILE *));
-static	void	clearvars	P((struct parse *, FILE *));
-static	void	showvars	P((struct parse *, FILE *));
-static	int dolist		P((struct varlist *, int, int, int, FILE *));
-static	void	readlist	P((struct parse *, FILE *));
-static	void	writelist	P((struct parse *, FILE *));
-static	void	readvar 	P((struct parse *, FILE *));
-static	void	writevar	P((struct parse *, FILE *));
-static	void	clocklist	P((struct parse *, FILE *));
-static	void	clockvar	P((struct parse *, FILE *));
-static	int findassidrange	P((u_int32, u_int32, int *, int *));
-static	void	mreadlist	P((struct parse *, FILE *));
-static	void	mreadvar	P((struct parse *, FILE *));
-static	int dogetassoc	P((FILE *));
-static	void	printassoc	P((int, FILE *));
-static	void	associations	P((struct parse *, FILE *));
-static	void	lassociations	P((struct parse *, FILE *));
-static	void	passociations	P((struct parse *, FILE *));
-static	void	lpassociations	P((struct parse *, FILE *));
+static	int checkassocid	(u_int32);
+static	char *	strsave 	(char *);
+static	struct varlist *findlistvar (struct varlist *, char *);
+static	void	doaddvlist	(struct varlist *, char *);
+static	void	dormvlist	(struct varlist *, char *);
+static	void	doclearvlist	(struct varlist *);
+static	void	makequerydata	(struct varlist *, int *, char *);
+static	int doquerylist (struct varlist *, int, int, int, u_short *, int *, char **);
+static	void	doprintvlist	(struct varlist *, FILE *);
+static	void	addvars 	(struct parse *, FILE *);
+static	void	rmvars		(struct parse *, FILE *);
+static	void	clearvars	(struct parse *, FILE *);
+static	void	showvars	(struct parse *, FILE *);
+static	int dolist		(struct varlist *, int, int, int, FILE *);
+static	void	readlist	(struct parse *, FILE *);
+static	void	writelist	(struct parse *, FILE *);
+static	void	readvar 	(struct parse *, FILE *);
+static	void	writevar	(struct parse *, FILE *);
+static	void	clocklist	(struct parse *, FILE *);
+static	void	clockvar	(struct parse *, FILE *);
+static	int findassidrange	(u_int32, u_int32, int *, int *);
+static	void	mreadlist	(struct parse *, FILE *);
+static	void	mreadvar	(struct parse *, FILE *);
+static	int dogetassoc	(FILE *);
+static	void	printassoc	(int, FILE *);
+static	void	associations	(struct parse *, FILE *);
+static	void	lassociations	(struct parse *, FILE *);
+static	void	passociations	(struct parse *, FILE *);
+static	void	lpassociations	(struct parse *, FILE *);
 
 #ifdef	UNUSED
-static	void	radiostatus P((struct parse *, FILE *));
+static	void	radiostatus (struct parse *, FILE *);
 #endif	/* UNUSED */
 
-static	void	pstatus 	P((struct parse *, FILE *));
-static	long	when		P((l_fp *, l_fp *, l_fp *));
-static	char *	prettyinterval	P((char *, long));
-static	int doprintpeers	P((struct varlist *, int, int, int, char *, FILE *, int));
-static	int dogetpeers	P((struct varlist *, int, FILE *, int));
-static	void	dopeers 	P((int, FILE *, int));
-static	void	peers		P((struct parse *, FILE *));
-static	void	lpeers		P((struct parse *, FILE *));
-static	void	doopeers	P((int, FILE *, int));
-static	void	opeers		P((struct parse *, FILE *));
-static	void	lopeers 	P((struct parse *, FILE *));
+static	void	pstatus 	(struct parse *, FILE *);
+static	long	when		(l_fp *, l_fp *, l_fp *);
+static	char *	prettyinterval	(char *, long);
+static	int doprintpeers	(struct varlist *, int, int, int, char *, FILE *, int);
+static	int dogetpeers	(struct varlist *, int, FILE *, int);
+static	void	dopeers 	(int, FILE *, int);
+static	void	peers		(struct parse *, FILE *);
+static	void	lpeers		(struct parse *, FILE *);
+static	void	doopeers	(int, FILE *, int);
+static	void	opeers		(struct parse *, FILE *);
+static	void	lopeers 	(struct parse *, FILE *);
+static  void    config          (struct parse *, FILE *);
+static  void    config_from_file (struct parse *, FILE *);
 
 
 /*
@@ -150,6 +152,12 @@ struct xcmd opcmds[] = {
 	{ "lopeers", lopeers,   { OPT|IP_VERSION, NO, NO, NO },
 	  { "-4|-6", "", "", "" },
 	  "obtain and print a list of all peers and clients showing dstadr [IP version]" },
+	{ ":config", config,   { NTP_STR, NO, NO, NO },
+	  { "<configuration command line>", "", "", "" },
+	  "send a remote configuration command to ntpd" },
+	{ "config-from-file", config_from_file, { NTP_STR, NO, NO, NO },
+	  { "<configuration filename>", "", "", "" },
+	  "configure ntpd using the configuration filename" },
 	{ 0,		0,		{ NO, NO, NO, NO },
 	  { "-4|-6", "", "", "" }, "" }
 };
@@ -158,23 +166,24 @@ struct xcmd opcmds[] = {
 /*
  * Variable list data space
  */
+#define MAXLINE     512  /* maximum length of a line */
 #define MAXLIST 	64	/* maximum number of variables in list */
 #define LENHOSTNAME 256 /* host name is 256 characters long */
 /*
  * Old CTL_PST defines for version 2.
  */
-#define OLD_CTL_PST_CONFIG			0x80
+#define OLD_CTL_PST_CONFIG		0x80
 #define OLD_CTL_PST_AUTHENABLE		0x40
 #define OLD_CTL_PST_AUTHENTIC		0x20
-#define OLD_CTL_PST_REACH			0x10
-#define OLD_CTL_PST_SANE			0x08
-#define OLD_CTL_PST_DISP			0x04
+#define OLD_CTL_PST_REACH		0x10
+#define OLD_CTL_PST_SANE		0x08
+#define OLD_CTL_PST_DISP		0x04
+
 #define OLD_CTL_PST_SEL_REJECT		0
 #define OLD_CTL_PST_SEL_SELCAND 	1
 #define OLD_CTL_PST_SEL_SYNCCAND	2
 #define OLD_CTL_PST_SEL_SYSPEER 	3
 
-
 char flash2[] = " .+*    "; /* flash decode for version 2 */
 char flash3[] = " x.-+#*o"; /* flash decode for peer status version 3 */
 
@@ -534,7 +543,7 @@ dolist(
 		return 1;
 	}
 
-	(void) fprintf(fp,"assID=%d ",associd);
+	(void) fprintf(fp,"associd=%d ",associd);
 	printvars(dsize, datap, (int)rstatus, type, fp);
 	return 1;
 }
@@ -602,7 +611,7 @@ writelist(
 	if (dsize == 0)
 		(void) fprintf(fp, "done! (no data returned)\n");
 	else {
-		(void) fprintf(fp,"assID=%d ",associd);
+		(void) fprintf(fp,"associd=%d ",associd);
 		printvars(dsize, datap, (int)rstatus,
 			  (associd != 0) ? TYPE_PEER : TYPE_SYS, fp);
 	}
@@ -677,7 +686,7 @@ writevar(
 	if (dsize == 0)
 		(void) fprintf(fp, "done! (no data returned)\n");
 	else {
-		(void) fprintf(fp,"assID=%d ",associd);
+		(void) fprintf(fp,"associd=%d ",associd);
 		printvars(dsize, datap, (int)rstatus,
 			  (associd != 0) ? TYPE_PEER : TYPE_SYS, fp);
 	}
@@ -759,8 +768,8 @@ findassidrange(
 	}
 
 	if (assid2 == 0 || assid2 > 65535) {
-		(void) fprintf(stderr,
-				   "***Invalid association ID %lu specified\n", (u_long)assid2);
+	fprintf(stderr,
+	    "***Invalid association ID %lu specified\n", (u_long)assid2);
 		return 0;
 	}
 
@@ -869,13 +878,13 @@ dogetassoc(
 	FILE *fp
 	)
 {
-	u_short *datap;
+	char *datap;
 	int res;
 	int dsize;
 	u_short rstatus;
 
 	res = doquery(CTL_OP_READSTAT, 0, 0, 0, (char *)0, &rstatus,
-			  &dsize, (char **)&datap);
+			  &dsize, &datap);
 
 	if (res != 0)
 		return 0;
@@ -898,10 +907,10 @@ dogetassoc(
 
 	numassoc = 0;
 	while (dsize > 0) {
-		assoc_cache[numassoc].assid = ntohs(*datap);
-		datap++;
-		assoc_cache[numassoc].status = ntohs(*datap);
-		datap++;
+		assoc_cache[numassoc].assid = ntohs(*((u_short *)datap));
+		datap += sizeof(u_short);
+		assoc_cache[numassoc].status = ntohs(*((u_short *)datap));
+		datap += sizeof(u_short);
 		if (++numassoc >= MAXASSOC)
 			break;
 		dsize -= sizeof(u_short) + sizeof(u_short);
@@ -942,7 +951,7 @@ printassoc(
 	 * Output a header
 	 */
 	(void) fprintf(fp,
-			   "\nind assID status  conf reach auth condition  last_event cnt\n");
+			   "\nind assid status  conf reach auth condition  last_event cnt\n");
 	(void) fprintf(fp,
 			   "===========================================================\n");
 	for (i = 0; i < numassoc; i++) {
@@ -955,103 +964,141 @@ printassoc(
 			conf = "yes";
 		else
 			conf = "no";
-		if (statval & CTL_PST_REACH || 1) {
-			reach = "yes";
+		if (statval & CTL_PST_BCAST) {
+			reach = "none";
+			if (statval & CTL_PST_AUTHENABLE)
+				auth = "yes";
+			else
+				auth = "none";
+		} else {
+			if (statval & CTL_PST_REACH)
+				reach = "yes";
+			else
+				reach = "no";
 			if (statval & CTL_PST_AUTHENABLE) {
 				if (statval & CTL_PST_AUTHENTIC)
 					auth = "ok ";
 				else
 					auth = "bad";
-			} else
+			} else {
 				auth = "none";
-
-			if (pktversion > NTP_OLDVERSION)
-				switch (statval & 0x7) {
-				case CTL_PST_SEL_REJECT:
-					condition = "reject";
-					break;
-				case CTL_PST_SEL_SANE:
-					condition = "falsetick";
-					break;
-				case CTL_PST_SEL_CORRECT:
-					condition = "excess";
-					break;
-				case CTL_PST_SEL_SELCAND:
-					condition = "outlyer";
-					break;
-				case CTL_PST_SEL_SYNCCAND:
-					condition = "candidat";
-					break;
-				case CTL_PST_SEL_DISTSYSPEER:
-					condition = "selected";
-					break;
-				case CTL_PST_SEL_SYSPEER:
-					condition = "sys.peer";
-					break;
-				case CTL_PST_SEL_PPS:
-					condition = "pps.peer";
-					break;
-				}
-			else
-				switch (statval & 0x3) {
-				case OLD_CTL_PST_SEL_REJECT:
-					if (!(statval & OLD_CTL_PST_SANE))
-					condition = "insane";
-					else if (!(statval & OLD_CTL_PST_DISP))
-					condition = "hi_disp";
-					else
-					condition = "";
-					break;
-				case OLD_CTL_PST_SEL_SELCAND:
-					condition = "sel_cand";
-					break;
-				case OLD_CTL_PST_SEL_SYNCCAND:
-					condition = "sync_cand";
-					break;
-				case OLD_CTL_PST_SEL_SYSPEER:
-					condition = "sys_peer";
-					break;
-				}
-
-		} else {
-			reach = "no";
-			auth = condition = "";
+			}
 		}
+		if (pktversion > NTP_OLDVERSION) {
+			switch (statval & 0x7) {
 
+			case CTL_PST_SEL_REJECT:
+				condition = "reject";
+				break;
+
+			case CTL_PST_SEL_SANE:
+				condition = "falsetick";
+				break;
+
+			case CTL_PST_SEL_CORRECT:
+				condition = "excess";
+				break;
+
+			case CTL_PST_SEL_SELCAND:
+				condition = "outlyer";
+				break;
+
+			case CTL_PST_SEL_SYNCCAND:
+				condition = "candidate";
+				break;
+
+			case CTL_PST_SEL_EXCESS:
+				condition = "backup";
+				break;
+
+			case CTL_PST_SEL_SYSPEER:
+				condition = "sys.peer";
+				break;
+
+			case CTL_PST_SEL_PPS:
+				condition = "pps.peer";
+				break;
+			}
+		} else {
+			switch (statval & 0x3) {
+
+			case OLD_CTL_PST_SEL_REJECT:
+				if (!(statval & OLD_CTL_PST_SANE))
+					condition = "insane";
+				else if (!(statval & OLD_CTL_PST_DISP))
+					condition = "hi_disp";
+				else
+					condition = "";
+				break;
+
+			case OLD_CTL_PST_SEL_SELCAND:
+				condition = "sel_cand";
+				break;
+
+			case OLD_CTL_PST_SEL_SYNCCAND:
+				condition = "sync_cand";
+				break;
+
+			case OLD_CTL_PST_SEL_SYSPEER:
+				condition = "sys_peer";
+				break;
+			}
+		}
 		switch (PEER_EVENT|event) {
-			case EVNT_PEERIPERR:
-			last_event = "IP error";
+
+		case PEVNT_MOBIL:
+			last_event = "mobilize";
 			break;
-			case EVNT_PEERAUTH:
-			last_event = "auth fail";
+
+		case PEVNT_DEMOBIL:
+			last_event = "demobilize";
 			break;
-			case EVNT_UNREACH:
-			last_event = "lost reach";
-			break;
-			case EVNT_REACH:
+
+		case PEVNT_REACH:
 			last_event = "reachable";
 			break;
-			case EVNT_PEERCLOCK:
-			last_event = "clock expt";
+
+		case PEVNT_UNREACH:
+			last_event = "unreachable";
 			break;
-#if 0
-			case EVNT_PEERSTRAT:
-			last_event = "stratum chg";
+
+		case PEVNT_RESTART:
+			last_event = "restart";
 			break;
-#endif
-			default:
+
+		case PEVNT_REPLY:
+			last_event = "no_reply";
+			break;
+
+		case PEVNT_RATE:
+			last_event = "raate_exceeded";
+			break;
+
+		case PEVNT_DENY:
+			last_event = "access_denied";
+			break;
+
+		case PEVNT_ARMED:
+			last_event = "leap_armed";
+			break;
+
+		case PEVNT_NEWPEER:
+			last_event = "sys_peer";
+			break;
+
+		case PEVNT_CLOCK:
+			last_event = "clock_alarm";
+			break;
+
+		default:
 			last_event = "";
 			break;
 		}
-
-		if (event_count != 0)
-			cnt = uinttoa(event_count);
-		else
-			cnt = "";
-		(void) sprintf(buf,
-				   "%3d %5u  %04x   %3.3s  %4s  %4.4s %9.9s %11s %2s",
-				   i+1, assoc_cache[i].assid, assoc_cache[i].status,
-				   conf, reach, auth, condition, last_event, cnt);
+		cnt = uinttoa(event_count);
+		sprintf(buf,
+		    "%3d %5u  %04x   %3.3s  %4s  %4.4s %9.9s %11s %2s",
+		    i + 1, assoc_cache[i].assid, assoc_cache[i].status,
+		    conf, reach, auth, condition, last_event, cnt);
 		bp = &buf[strlen(buf)];
 		while (bp > buf && *(bp-1) == ' ')
 			*(--bp) = '\0';
@@ -1060,7 +1107,6 @@ printassoc(
 }
 
 
-
 /*
  * associations - get, record and print a list of associations
  */
@@ -1186,7 +1232,7 @@ pstatus(
 		return;
 	}
 
-	(void) fprintf(fp,"assID=%d ",associd);
+	(void) fprintf(fp,"associd=%d ",associd);
 	printvars(dsize, datap, (int)rstatus, TYPE_PEER, fp);
 }
 
@@ -1253,17 +1299,15 @@ prettyinterval(
 
 static char
 decodeaddrtype(
-	struct sockaddr_storage *sock
+	sockaddr_u *sock
 	)
 {
 	char ch = '-';
 	u_int32 dummy;
-	struct sockaddr_in6 *sin6;
 
-	switch(sock->ss_family) {
+	switch(AF(sock)) {
 	case AF_INET:
-		dummy = ((struct sockaddr_in *)sock)->sin_addr.s_addr;
-		dummy = ntohl(dummy);
+		dummy = SRCADR(sock);
 		ch = (char)(((dummy&0xf0000000)==0xe0000000) ? 'm' :
 			((dummy&0x000000ff)==0x000000ff) ? 'b' :
 			((dummy&0xffffffff)==0x7f000001) ? 'l' :
@@ -1271,8 +1315,7 @@ decodeaddrtype(
 			'u');
 		break;
 	case AF_INET6:
-		sin6 = (struct sockaddr_in6 *)sock;
-		if (IN6_IS_ADDR_MULTICAST(&sin6->sin6_addr))
+		if (IN6_IS_ADDR_MULTICAST(PSOCK_ADDR6(sock)))
 			ch = 'm';
 		else
 			ch = 'u';
@@ -1356,8 +1399,8 @@ doprintpeers(
 	int i;
 	int c;
 
-	struct sockaddr_storage srcadr;
-	struct sockaddr_storage dstadr;
+	sockaddr_u srcadr;
+	sockaddr_u dstadr;
 	u_long srcport = 0;
 	char *dstadr_refid = "0.0.0.0";
 	u_long stratum = 0;
@@ -1381,8 +1424,8 @@ doprintpeers(
 	memset((char *)havevar, 0, sizeof(havevar));
 	get_systime(&ts);
 	
-	memset((char *)&srcadr, 0, sizeof(struct sockaddr_storage));
-	memset((char *)&dstadr, 0, sizeof(struct sockaddr_storage));
+	ZERO_SOCK(&srcadr);
+	ZERO_SOCK(&dstadr);
 
 	/* Initialize by zeroing out estimate variables */
 	memset((char *)&estoffset, 0, sizeof(l_fp));
@@ -1391,22 +1434,23 @@ doprintpeers(
 	memset((char *)&estdisp, 0, sizeof(l_fp));
 
 	while (nextvar(&datalen, &data, &name, &value)) {
-		struct sockaddr_storage dum_store;
+		sockaddr_u dum_store;
 
 		i = findvar(name, peer_var, 1);
 		if (i == 0)
 			continue;	/* don't know this one */
 		switch (i) {
 			case CP_SRCADR:
-			if (decodenetnum(value, &srcadr))
+			if (decodenetnum(value, &srcadr)) {
 				havevar[HAVE_SRCADR] = 1;
+			}
 			break;
 			case CP_DSTADR:
-			if (decodenetnum(value, &dum_store))
+			if (decodenetnum(value, &dum_store)) {
 				type = decodeaddrtype(&dum_store);
-			if (pvl == opeervarlist) {
-				if (decodenetnum(value, &dstadr)) {
+				if (pvl == opeervarlist) {
 					havevar[HAVE_DSTADR] = 1;
+					dstadr = dum_store;
 					dstadr_refid = stoa(&dstadr);
 				}
 			}
@@ -1416,16 +1460,6 @@ doprintpeers(
 				havevar[HAVE_REFID] = 1;
 				if (*value == '\0') {
 					dstadr_refid = "0.0.0.0";
-				} else if (decodenetnum(value, &dstadr)) {
-					if (SOCKNUL(&dstadr))
-						dstadr_refid = "0.0.0.0";
-					else if ((dstadr.ss_family == AF_INET)
-					    && ISREFCLOCKADR(&dstadr))
-    						dstadr_refid =
-						    refnumtoa(&dstadr);
-					else
-						dstadr_refid =
-						    stoa(&dstadr);
 				} else if ((int)strlen(value) <= 4) {
 					refid_string[0] = '.';
 					(void) strcpy(&refid_string[1], value);
@@ -1433,6 +1467,15 @@ doprintpeers(
 					refid_string[i] = '.';
 					refid_string[i+1] = '\0';
 					dstadr_refid = refid_string;
+				} else if (decodenetnum(value, &dstadr)) {
+					if (SOCK_UNSPEC(&dstadr))
+						dstadr_refid = "0.0.0.0";
+					else if (ISREFCLOCKADR(&dstadr))
+						dstadr_refid =
+						    refnumtoa(&dstadr);
+					else
+						dstadr_refid =
+						    stoa(&dstadr);
 				} else {
 					havevar[HAVE_REFID] = 0;
 				}
@@ -1469,8 +1512,9 @@ doprintpeers(
 				havevar[HAVE_OFFSET] = 1;
 			break;
 			case CP_JITTER:
-			if (decodetime(value, &estjitter))
-				havevar[HAVE_JITTER] = 1;
+			if (pvl == peervarlist)
+				if (decodetime(value, &estjitter))
+					havevar[HAVE_JITTER] = 1;
 			break;
 			case CP_DISPERSION:
 			if (decodetime(value, &estdisp))
@@ -1511,7 +1555,7 @@ doprintpeers(
 		c = flash2[CTL_PEER_STATVAL(rstatus) & 0x3];
 	if (numhosts > 1)
 		(void) fprintf(fp, "%-*s ", maxhostlen, currenthost);
-	if (af == 0 || srcadr.ss_family == af){
+	if (af == 0 || AF(&srcadr) == af) {
 		strcpy(clock_name, nntohost(&srcadr));
 		
 		(void) fprintf(fp,
@@ -1601,7 +1645,7 @@ dopeers(
 {
 	register int i;
 	char fullname[LENHOSTNAME];
-	struct sockaddr_storage netnum;
+	sockaddr_u netnum;
 
 	if (!dogetassoc(fp))
 		return;
@@ -1690,7 +1734,7 @@ doopeers(
 {
 	register int i;
 	char fullname[LENHOSTNAME];
-	struct sockaddr_storage netnum;
+	sockaddr_u netnum;
 
 	if (!dogetassoc(fp))
 		return;
@@ -1765,3 +1809,84 @@ lopeers(
 	}
 	doopeers(1, fp, af);
 }
+
+
+/* 
+ * config - send a configuration command to a remote host
+ */
+static void 
+config (
+	struct parse *pcmd,
+	FILE *fp
+	)
+{
+    u_short rstatus;
+    int rsize;
+    char *rdata;
+    int res;
+
+    if (debug > 2) {
+        printf("In Config\n");
+        printf("Keyword = %s\n", pcmd->keyword);
+        printf("Command = %s\n", pcmd->argval[0].string);
+    }
+    
+    res = doquery(CTL_OP_CONFIGURE, 0, 1, strlen(pcmd->argval[0].string),
+                  pcmd->argval[0].string, &rstatus,
+                  &rsize, &rdata);
+    rdata[rsize] = '\0';
+    printf("%s\n", rdata);
+
+}
+
+/* 
+ * config_from_file - remotely configure an ntpd daemon using the
+ * specified configuration file
+ * SK: This function is a kludge at best and is full of bad design
+ * bugs:
+ * 1. ntpq uses UDP, which means that there is no guarantee of in-order,
+ *    error-free delivery. 
+ * 2. The maximum length of a packet is constrained, and as a result, the
+ *    maximum length of a line in a configuration file is constrained. 
+ *    Longer lines will lead to unpredictable results.
+ * 3. Since this function is sending a line at a time, we can't update
+ *    the control key through the configuration file (YUCK!!)
+ */
+static void 
+config_from_file (
+	struct parse *pcmd,
+	FILE *fp
+	)
+{
+    u_short rstatus;
+    int rsize;
+    char *rdata;
+    int res;
+    FILE *config_fd;
+    char config_cmd[MAXLINE];
+
+    if (debug > 2) {
+        printf("In Config\n");
+        printf("Keyword = %s\n", pcmd->keyword);
+        printf("Filename = %s\n", pcmd->argval[0].string);
+    }
+
+    if ((config_fd = fopen(pcmd->argval[0].string, "r")) == NULL) {
+        printf("ERROR!! Couldn't open file: %s\n", pcmd->argval[0].string);
+    } 
+    else {
+        int i = 0;
+        printf("Sending configuration file, one line at a time.\n");
+        while (fgets(config_cmd, MAXLINE, config_fd) != NULL) {
+            ++i;
+            res = doquery(CTL_OP_CONFIGURE, 0, 1, strlen(config_cmd),
+                          config_cmd, &rstatus,
+                          &rsize, &rdata);
+            printf("Line No: %d ", i);
+            rdata[rsize] = '\0';
+            printf(rdata);
+        }
+        printf("Done sending file\n");
+        fclose(config_fd);
+    }
+}
